#ifndef FSGUISTATUSBAR_IS_INCLUDED
#define FSGUISTATUSBAR_IS_INCLUDED
/* { */

#include "fsguibase.h"

/*! This class is a status bar, which can be placed at the top and the bottom of the GUI canvas. 
    */
class FsGuiStatusBar : public FsGuiObject
{
private:
	class Cell : public FsGuiObject
	{
	public:
		YsWString msg;
		mutable FsGuiBitmapType bmpCache;
		void CacheBitmap(void);
	};

	int widthInPixel;
	int heightInPixel;
	YsSegmentedArray <Cell,3> columnArray;
	YsColor bgColor;
	mutable YSBOOL needRedraw;
public:
	/*! Default constructor. */
	FsGuiStatusBar();

	/*! This function clears the object.  
	    The status bar must be re-constructed when this object is recycled. */
	void CleanUp(void);

	/*! This function draws the status bar.  y0 is the y coordinate of the top-left corner of the status bar. */
	void Show(int y0) const;

	/*! This function creates empty columns.  */
	void Make(int nColumn,int widthInPixel);

	/*! This function sets the width (in pixels) of the status bar. */
	void SetWidth(int widthInPixel);

	/*! This function returns the background color. */
	const YsColor &GetBackgroundColor(void) const;

	/*! This function changes the background color. */
	void SetBackgroundColor(const YsColor &newBackgroundColor);

	/*! This function clears all messages in the status bar. */
	void ClearStringAll(void);

	/*! This function sets the message for the specified column.  Column index (columnId) is zero based. */
	YSRESULT SetString(int columnId,const wchar_t msg[]);

	/*! This function sets the message for the specified column.  Column index (columnId) is zero based. */
	YSRESULT SetString(int columnId,const char msg[]);

	/*! This function copies the string of the specified column (columnIdx) in msg and returns the pointer to msg. */
	const wchar_t *GetString(YsWString &msg,int columnIdx) const;

	/*! This function copies the string of the specified column (columnIdx) in msg in UTF8 format and returns the pointer to msg. */
	const char *GetString(YsString &msg,int columnIdx) const;

	/*! This function returns the height in the number of pixels. */
	int GetHeight(void) const;

	/*! This function sets or clears the needRedraw flag. */
	virtual void SetNeedRedraw(YSBOOL needRedraw);

	/*! This function returns the needRedraw flag. */
	virtual YSBOOL NeedRedraw(void) const;
};

/* } */
#endif
