/* ////////////////////////////////////////////////////////////

  File Name: fsgui3dapp_draw.cpp
  Copyright (c) 2014 Soji Yamakawa.  All rights reserved.
  http://www.ysflight.com

//////////////////////////////////////////////////////////// */

#include "fsgui3dapp.h"
#include "miscdlg.h"
#include "refbmp/refbmp.h"
#include <ysgl.h>
#include "ysglbuffer.h"
#include "config/config.h"


void FsGui3DMainCanvas::Draw(void)
{
	// Do this at the beginning of Draw funtion.  This will allow one of the elements set SetNeedRedraw(YSTRUE) 
	// within drawing function so that Draw function will be called again in the next iteragion. >>
	SetNeedRedraw(YSFALSE);
	drawEnv.SetNeedRedraw(YSFALSE);
	threeDInterface.SetNeedRedraw(YSFALSE);
	// <<


	glEnable(GL_POLYGON_OFFSET_FILL);
	glPolygonOffset(1,1);


	if(NULL==slHd)
	{
		if(NULL!=GetTopStatusBar())
		{
			GetTopStatusBar()->ClearStringAll();
		}
		if(NULL!=GetBottomStatusBar())
		{
			GetBottomStatusBar()->ClearStringAll();
		}
	}

	if(NULL!=slHd && NULL!=GetTopStatusBar())
	{
		YsWString ful,pth,fil;
		slHd->GetFileName(ful);
		ful.SeparatePathFile(pth,fil);
		if(YSTRUE==slHd->IsModified())
		{
			fil.Append(L"(*)");
		}

		YsWString curMsg;
		if(0!=YsWString::Strcmp(fil,GetTopStatusBar()->GetString(curMsg,0)))
		{
			GetTopStatusBar()->SetString(0,fil);
		}
	}

	if(NULL!=slHd && NULL!=GetBottomStatusBar())
	{
		YsString str("Selection");
		YsString append;

		{
			YsArray <YsShellVertexHandle> selVtHd;
			slHd->GetSelectedVertex(selVtHd);
			if(0<selVtHd.GetN())
			{
				append.Printf("  Vertex:%d",(int)selVtHd.GetN());
				str.Append(append);
			}
		}
		{
			YsArray <YsShellPolygonHandle> selPlHd;
			slHd->GetSelectedPolygon(selPlHd);
			if(0<selPlHd.GetN())
			{
				append.Printf("  Polygon:%d",(int)selPlHd.GetN());
				str.Append(append);
			}
		}
		{
			YsArray <YsShellExt::ConstEdgeHandle> selCeHd;
			slHd->GetSelectedConstEdge(selCeHd);
			if(0<selCeHd.GetN())
			{
				append.Printf("  ConstEdge:%d",(int)selCeHd.GetN());
				str.Append(append);
			}
		}
		{
			YsArray <YsShellExt::FaceGroupHandle> selFgHd;
			slHd->GetSelectedFaceGroup(selFgHd);
			if(0<selFgHd.GetN())
			{
				append.Printf("  FaceGroup:%d",(int)selFgHd.GetN());
				str.Append(append);
			}
		}
		GetBottomStatusBar()->SetString(0,str);
	}


	int viewport[4];
	drawEnv.GetOpenGlCompatibleViewport(viewport);
	drawEnv.SetVerticalOrientation(YSTRUE);
	drawEnv.UpdateNearFar();


	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LEQUAL);


	if(YsCoordSysModel==YSOPENGL)
	{
		glFrontFace(GL_CCW);
	}
	else
	{
		glFrontFace(GL_CW);
	}


	const YsMatrix4x4 &projMat=drawEnv.GetProjectionMatrix();
	const YsMatrix4x4 &viewMat=drawEnv.GetViewMatrix();
	const YsMatrix4x4 projViewMat=(projMat*viewMat);

	threeDInterface.SetViewport(viewport);
	threeDInterface.SetProjViewModelMatrix(projViewMat);


	glViewport(viewport[0],viewport[1],viewport[2],viewport[3]);

	GLfloat glProjMat[16];
	projMat.GetOpenGlCompatibleMatrix(glProjMat);

	GLfloat glModelviewMat[16];
	viewMat.GetOpenGlCompatibleMatrix(glModelviewMat);


	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glMultMatrixf(glProjMat);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glMultMatrixf(glModelviewMat);


	YsGLSLSetShared3DRendererProjection(glProjMat);
	YsGLSLSetShared3DRendererModelView(glModelviewMat);


	actuallyDrawVertex=drawVertex;
	actuallyDrawConstEdge=drawConstEdge;
	actuallyDrawPolygonEdge=drawPolygonEdge;
	actuallyDrawPolygon=drawPolygon;
	actuallyDrawShrunkPolygon=drawShrunkPolygon;
	actuallyDrawNonManifoldEdge=drawNonManifoldEdge;

	if(NULL!=UIBeforeDrawCallBack)
	{
		(*UIBeforeDrawCallBack)(*this);
	}

	glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);



	{
// Linear interpolation looks to be too fast.  Probably need three colors.
		YsGLSLUsePlain2DRenderer(YsGLSLSharedPlain2DRenderer());
		YsGLSLUseWindowCoordinateInPlain2DDrawing(YsGLSLSharedPlain2DRenderer(),YSTRUE);

		glDepthMask(0);

		const GLfloat rect[12]=
		{
			0                   ,0,
			(GLfloat)viewport[2],0,
			0                   ,(GLfloat)viewport[3]/2.0f,
			(GLfloat)viewport[2],(GLfloat)viewport[3]/2.0f,
			0                   ,(GLfloat)viewport[3],
			(GLfloat)viewport[2],(GLfloat)viewport[3]
		};
		const GLfloat color[24]=
		{
			config->bgColor[0].Rf(),config->bgColor[0].Gf(),config->bgColor[0].Bf(),1,
			config->bgColor[0].Rf(),config->bgColor[0].Gf(),config->bgColor[0].Bf(),1,
			config->bgColor[1].Rf(),config->bgColor[1].Gf(),config->bgColor[1].Bf(),1,
			config->bgColor[1].Rf(),config->bgColor[1].Gf(),config->bgColor[1].Bf(),1,
			config->bgColor[2].Rf(),config->bgColor[2].Gf(),config->bgColor[2].Bf(),1,
			config->bgColor[2].Rf(),config->bgColor[2].Gf(),config->bgColor[2].Bf(),1
		};
		YsGLSLDrawPlain2DPrimitivefv(YsGLSLSharedPlain2DRenderer(),GL_TRIANGLE_STRIP,6,rect,color);

		glDepthMask(1);

		YsGLSLEndUsePlain2DRenderer(YsGLSLSharedPlain2DRenderer());
	}



	threeDInterface.Draw();


	if(NEED_REMAKE_DRAWING_NONE!=needRemakeDrawingBuffer)
	{
		RemakeDrawingBuffer(slHd,needRemakeDrawingBuffer);
		needRemakeDrawingBuffer=NEED_REMAKE_DRAWING_NONE;
	}
	if(NEED_REMAKE_DRAWING_NONE!=needRemakeDrawingBuffer_for_OtherShell && YSTRUE==actuallyDrawOtherShell)
	{
		YsArray <YsShellDnmContainer <YsShellExtEdit>::Node *> allNode;
		shlGrp.GetNodePointerAll(allNode);
		for(YSSIZE_T nodeIdx=0; nodeIdx<allNode.GetN(); ++nodeIdx)
		{
			if(slHd!=allNode[nodeIdx])
			{
				RemakeDrawingBuffer(allNode[nodeIdx],needRemakeDrawingBuffer_for_OtherShell);
			}
		}
		needRemakeDrawingBuffer_for_OtherShell=NEED_REMAKE_DRAWING_NONE;
	}

	YsShellExtDrawingBuffer &drawBuf=slHd->GetDrawingBuffer();

	if(YSTRUE==actuallyDrawVertex && NULL!=slHd)
	{
		struct YsGLSL3DRenderer *renderer=YsGLSLSharedMonoColorMarker3DRenderer();
		YsShellExtDrawingBuffer &buf=slHd->GetDrawingBuffer();
		YsGLSLUse3DRenderer(renderer);

		const GLfloat vtxCol[4]={1.0,0.0,1.0,1.0};
		YsGLSLSet3DRendererUniformColorfv(renderer,vtxCol);
		if(0<buf.normalVertexPosBuffer.GetNumVertex())
		{
			YsGLSLSet3DRendererUniformMarkerDimension(renderer,normalVertexMarkerRadius);
			YsGLSLSet3DRendererUniformMarkerType(renderer,YSGLSL_MARKER_TYPE_RECT);
			YsGLSLSet3DRendererViewportDimensionf(renderer,(GLfloat)viewport[2],(GLfloat)viewport[3]);
			YsGLSLDrawPrimitiveVtxPixelOffsetfv(renderer,GL_TRIANGLES,buf.normalVertexPosBuffer.GetNumVertex(),buf.normalVertexPosBuffer,buf.normalVertexPixOffsetBuffer);
		}
		if(0<buf.roundVertexPosBuffer.GetNumVertex())
		{
			YsGLSLSet3DRendererUniformMarkerDimension(renderer,roundVertexMarkerRadius);
			YsGLSLSet3DRendererUniformMarkerType(renderer,YSGLSL_MARKER_TYPE_CIRCLE);
			YsGLSLSet3DRendererViewportDimensionf(renderer,(GLfloat)viewport[2],(GLfloat)viewport[3]);
			YsGLSLDrawPrimitiveVtxPixelOffsetfv(renderer,GL_TRIANGLES,buf.roundVertexPosBuffer.GetNumVertex(),buf.roundVertexPosBuffer,buf.roundVertexPixOffsetBuffer);
		}

		YsGLSLEndUse3DRenderer(renderer);
	}

	if(0<drawBuf.selectedVertexPosBuffer.GetNumVertex() && NULL!=slHd)
	{
		struct YsGLSL3DRenderer *renderer=YsGLSLSharedMonoColorMarker3DRenderer();
		YsShellExtDrawingBuffer &buf=slHd->GetDrawingBuffer();
		YsGLSLUse3DRenderer(renderer);

		const GLfloat selVtxCol[4]={0.0,0.0,1.0,1.0};
		YsGLSLSet3DRendererUniformColorfv(renderer,selVtxCol);
		YsGLSLSet3DRendererUniformMarkerDimension(renderer,selectedVertexMarkerRadius);
		YsGLSLSet3DRendererUniformMarkerType(renderer,YSGLSL_MARKER_TYPE_EMPTY_RECT);
		YsGLSLSet3DRendererViewportDimensionf(renderer,(GLfloat)viewport[2],(GLfloat)viewport[3]);
		YsGLSLDrawPrimitiveVtxPixelOffsetfv(renderer,GL_TRIANGLES,buf.selectedVertexPosBuffer.GetNumVertex(),buf.selectedVertexPosBuffer,buf.selectedVertexPixOffsetBuffer);

		YsGLSLEndUse3DRenderer(renderer);
	}

	if(0<drawBuf.selectedVertexLineBuffer.GetNumVertex() && NULL!=slHd)
	{
		struct YsGLSL3DRenderer *renderer=YsGLSLSharedVariColor3DRenderer();
		YsShellExtDrawingBuffer &buf=slHd->GetDrawingBuffer();
		YsGLSLUse3DRenderer(renderer);
		YsGLSLDrawPrimitiveVtxColfv(renderer,GL_LINES,buf.selectedVertexLineBuffer.GetNumVertex(),buf.selectedVertexLineBuffer,buf.selectedVertexLineColBuffer);
		YsGLSLEndUse3DRenderer(renderer);
	}

	if(YSTRUE==actuallyDrawPolygonEdge && NULL!=slHd)
	{
		struct YsGLSL3DRenderer *renderer=YsGLSLSharedFlat3DRenderer();
		YsShellExtDrawingBuffer &buf=slHd->GetDrawingBuffer();
		YsGLSLUse3DRenderer(renderer);

		const GLfloat edgeCol[4]={0,0,0,1.0f};
		YsGLSLSet3DRendererUniformColorfv(renderer,edgeCol);
		YsGLSLDrawPrimitiveVtxfv(renderer,GL_LINES,buf.normalEdgePosBuffer.GetNumVertex(),buf.normalEdgePosBuffer);

		YsGLSLEndUse3DRenderer(renderer);
	}

	if(YSTRUE==actuallyDrawShrunkPolygon && NULL!=slHd)
	{
		struct YsGLSL3DRenderer *renderer=YsGLSLSharedFlat3DRenderer();
		YsShellExtDrawingBuffer &buf=slHd->GetDrawingBuffer();
		YsGLSLUse3DRenderer(renderer);

		const GLfloat edgeCol[4]={0,0,0,1.0f};
		YsGLSLSet3DRendererUniformColorfv(renderer,edgeCol);
		YsGLSLDrawPrimitiveVtxfv(renderer,GL_LINES,buf.shrunkEdgePosBuffer.GetNumVertex(),buf.shrunkEdgePosBuffer);

		YsGLSLEndUse3DRenderer(renderer);
	}

	if(0<drawBuf.selectedPolygonPosBuffer.GetNumVertex() && NULL!=slHd)
	{
		YsGLSL3DRenderer *renderer=YsGLSLSharedVariColor3DRenderer();
		YsGLSLUse3DRenderer(renderer);
		glLineWidth(3.0f);

		YsGLSLDrawPrimitiveVtxColfv(renderer,GL_LINES,drawBuf.selectedPolygonPosBuffer.GetNumVertex(),drawBuf.selectedPolygonPosBuffer,drawBuf.selectedPolygonColBuffer);

		glLineWidth(1.0f);
		YsGLSLEndUse3DRenderer(renderer);
	}

	if(YSTRUE==actuallyDrawConstEdge && 0<drawBuf.constEdgeVtxBuffer.GetNumVertex() && NULL!=slHd)
	{
		struct YsGLSL3DRenderer *renderer=YsGLSLSharedFlat3DRenderer();
		YsShellExtDrawingBuffer &buf=slHd->GetDrawingBuffer();
		YsGLSLUse3DRenderer(renderer);
		glLineWidth(3);

		const GLfloat constEdgeCol[4]={0.0,0.0,0.0,1.0};
		YsGLSLSet3DRendererUniformColorfv(renderer,constEdgeCol);
		YsGLSLDrawPrimitiveVtxfv(renderer,GL_LINES,buf.constEdgeVtxBuffer.GetNumVertex(),buf.constEdgeVtxBuffer);

		glLineWidth(1);
		YsGLSLEndUse3DRenderer(renderer);
	}

	if(0<drawBuf.selectedConstEdgeVtxBuffer.GetNumVertex() && NULL!=slHd)
	{
		struct YsGLSL3DRenderer *renderer=YsGLSLSharedFlat3DRenderer();
		YsShellExtDrawingBuffer &buf=slHd->GetDrawingBuffer();
		YsGLSLUse3DRenderer(renderer);
		glLineWidth(4);

		const GLfloat selectedConstEdgeCol[4]={1.0,0.0,1.0,1.0};
		YsGLSLSet3DRendererUniformColorfv(renderer,selectedConstEdgeCol);
		YsGLSLDrawPrimitiveVtxfv(renderer,GL_LINES,buf.selectedConstEdgeVtxBuffer.GetNumVertex(),buf.selectedConstEdgeVtxBuffer);
		YsGLSLDrawPrimitiveVtxfv(renderer,GL_TRIANGLES,buf.selectedConstEdgeArrowHeadVtxBuffer.GetNumVertex(),buf.selectedConstEdgeArrowHeadVtxBuffer);

		glLineWidth(1);
		YsGLSLEndUse3DRenderer(renderer);
	}

	if(0<=drawBuf.selectedFaceGroupVtxBuffer.GetNumVertex() && NULL!=slHd)
	{
		struct YsGLSL3DRenderer *renderer=YsGLSLSharedFlat3DRenderer();
		YsShellExtDrawingBuffer &buf=slHd->GetDrawingBuffer();
		YsGLSLUse3DRenderer(renderer);

		const GLfloat selectedFaceGroupCol[4]={0.0,1.0,1.0,1.0};
		YsGLSLSet3DRendererUniformColorfv(renderer,selectedFaceGroupCol);
		YsGLSLDrawPrimitiveVtxfv(renderer,GL_TRIANGLES,buf.selectedFaceGroupVtxBuffer.GetNumVertex(),buf.selectedFaceGroupVtxBuffer);

		YsGLSLEndUse3DRenderer(renderer);
	}

	if(YSTRUE!=dnmEditMode)
	{
		DrawInGeometryEditMode();
	}
	else
	{
		DrawInDnmEditMode();
	}

	if(NULL!=slHd && YSTRUE==actuallyDrawNonManifoldEdge)
	{
		if(YSTRUE==nonManifoldEdgeCache.IsModified(slHd))
		{
			nonManifoldEdgeCache.CacheNonManifoldEdge(slHd);
		}
		struct YsGLSL3DRenderer *renderer=YsGLSLSharedVariColor3DRenderer();
		YsGLSLUse3DRenderer(renderer);

		glLineWidth(3);
		YsGLSLDrawPrimitiveVtxColfv(renderer,GL_LINES,nonManifoldEdgeCache.lineVtxBuf.GetNumVertex(),nonManifoldEdgeCache.lineVtxBuf,nonManifoldEdgeCache.lineColBuf);
		glLineWidth(1);

		YsGLSLEndUse3DRenderer(renderer);

		// The following could have easily be done by glPointSize.  However, stupid OpenGL ES 2.0 got rid of glPointSize.
		// OpenGL ES should be gone.  The hardware on the tablets and smart phones are good enough to support full-blown
		// OpenGL.  OpenGL ES will be remembered as a dark history of OpenGL.  That is bad.  Not as bad as Direct 3D though.
		renderer=YsGLSLSharedVariColorMarker3DRenderer();
		YsGLSLUse3DRenderer(renderer);

		YsGLSLSet3DRendererViewportDimensionf(renderer,(GLfloat)viewport[2],(GLfloat)viewport[3]);
		YsGLSLSet3DRendererUniformMarkerType(renderer,YSGLSL_MARKER_TYPE_RECT);
		YsGLSLSet3DRendererUniformMarkerDimension(renderer,1.5);
		YsGLSLDrawPrimitiveVtxColPixelOffsetfv(
		    renderer,GL_TRIANGLES,
		    nonManifoldEdgeCache.pointVtxBuf.GetNumVertex(),
		    nonManifoldEdgeCache.pointVtxBuf,
		    nonManifoldEdgeCache.pointColBuf,
		    nonManifoldEdgeCache.pointPixOffsetBuf);
		YsGLSLEndUse3DRenderer(renderer);
	}


	if(NULL!=UIDrawCallBack3D)
	{
		UIDrawCallBack3D(*this);
	}


	refBmpStore->Draw();


	if(YSTRUE==drawAxis)
	{
		const GLfloat axisLength=5.0;

		const GLfloat axisLine[6*3]=
		{
			0,0,0,  axisLength,0,0,
			0,0,0,  0,axisLength,0,
			0,0,0,  0,0,axisLength
		};
		const GLfloat axisColor[4]=
		{
			0,0,0,1.0f
		};

		struct YsGLSL3DRenderer *renderer=YsGLSLSharedFlat3DRenderer();
		YsGLSLUse3DRenderer(renderer);

		YsGLSLSet3DRendererUniformColorfv(renderer,axisColor);
		YsGLSLDrawPrimitiveVtxfv(renderer,GL_LINES,6,axisLine);

		int nTri=0;
		GLfloat triCoord[3*3*3];
		YsVec3 tri[3];
		if(YSOK==FsGui3DGetArrowHeadTriangle(tri,YsOrigin(),axisLength*YsXVec(),viewport,projMat,viewMat,0.1,10,8))
		{
			triCoord[nTri*9  ]=(GLfloat)tri[0].x();
			triCoord[nTri*9+1]=(GLfloat)tri[0].y();
			triCoord[nTri*9+2]=(GLfloat)tri[0].z();
			triCoord[nTri*9+3]=(GLfloat)tri[1].x();
			triCoord[nTri*9+4]=(GLfloat)tri[1].y();
			triCoord[nTri*9+5]=(GLfloat)tri[1].z();
			triCoord[nTri*9+6]=(GLfloat)tri[2].x();
			triCoord[nTri*9+7]=(GLfloat)tri[2].y();
			triCoord[nTri*9+8]=(GLfloat)tri[2].z();
			++nTri;
		}
		if(YSOK==FsGui3DGetArrowHeadTriangle(tri,YsOrigin(),axisLength*YsYVec(),viewport,projMat,viewMat,0.1,10,8))
		{
			triCoord[nTri*9  ]=(GLfloat)tri[0].x();
			triCoord[nTri*9+1]=(GLfloat)tri[0].y();
			triCoord[nTri*9+2]=(GLfloat)tri[0].z();
			triCoord[nTri*9+3]=(GLfloat)tri[1].x();
			triCoord[nTri*9+4]=(GLfloat)tri[1].y();
			triCoord[nTri*9+5]=(GLfloat)tri[1].z();
			triCoord[nTri*9+6]=(GLfloat)tri[2].x();
			triCoord[nTri*9+7]=(GLfloat)tri[2].y();
			triCoord[nTri*9+8]=(GLfloat)tri[2].z();
			++nTri;
		}
		if(YSOK==FsGui3DGetArrowHeadTriangle(tri,YsOrigin(),axisLength*YsZVec(),viewport,projMat,viewMat,0.1,10,8))
		{
			triCoord[nTri*9  ]=(GLfloat)tri[0].x();
			triCoord[nTri*9+1]=(GLfloat)tri[0].y();
			triCoord[nTri*9+2]=(GLfloat)tri[0].z();
			triCoord[nTri*9+3]=(GLfloat)tri[1].x();
			triCoord[nTri*9+4]=(GLfloat)tri[1].y();
			triCoord[nTri*9+5]=(GLfloat)tri[1].z();
			triCoord[nTri*9+6]=(GLfloat)tri[2].x();
			triCoord[nTri*9+7]=(GLfloat)tri[2].y();
			triCoord[nTri*9+8]=(GLfloat)tri[2].z();
			++nTri;
		}

		if(0<nTri)
		{
			YsGLSLDrawPrimitiveVtxfv(renderer,GL_TRIANGLES,nTri*3,triCoord);
		}


		YsGLSLEndUse3DRenderer(renderer);
	}



	// 2D Drawing
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(0.0,(double)viewport[2],(double)viewport[3],0.0,-1.0,1.0);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	YsGLSLUsePlain2DRenderer(YsGLSLSharedPlain2DRenderer());
	YsGLSLUseWindowCoordinateInPlain2DDrawing(YsGLSLSharedPlain2DRenderer(),YSTRUE);
	{
		if(YSTRUE==lBtnDown)
		{
			const GLfloat rect[8]=
			{
				(GLfloat)lBtnDownMx,(GLfloat)lBtnDownMy,
				(GLfloat)moveMx,    (GLfloat)lBtnDownMy,
				(GLfloat)moveMx,    (GLfloat)moveMy,
				(GLfloat)lBtnDownMx,(GLfloat)moveMy
			};
			const GLfloat color[16]=
			{
				0,0,0,0.8f,
				0,0,0,0.8f,
				0,0,0,0.8f,
				0,0,0,0.8f
			};
			YsGLSLDrawPlain2DPrimitivefv(YsGLSLSharedPlain2DRenderer(),GL_LINE_LOOP,4,rect,color);
		}
	}
	YsGLSLEndUsePlain2DRenderer(YsGLSLSharedPlain2DRenderer());

	glDisable(GL_DEPTH_TEST);

	FsGuiCanvas::Show();

	if(NULL!=UIDrawCallBack2D)
	{
		(*UIDrawCallBack2D)(*this);
	}

	FsSwapBuffers();
}

void FsGui3DMainCanvas::DrawInGeometryEditMode(void)
{
	if(YSTRUE==actuallyDrawPolygon && NULL!=slHd)
	{
		YsShellExtDrawingBuffer &buf=slHd->GetDrawingBuffer();

		glEnable(GL_CULL_FACE);

		if(NULL!=YsGLSLSharedVariColorShaded3DRenderer())
		{
			YsGLSLUse3DRenderer(YsGLSLSharedVariColorShaded3DRenderer());
			YsGLSLDrawPrimitiveVtxNomColfv(
			    YsGLSLSharedVariColorShaded3DRenderer(),
			    GL_TRIANGLES,
			    buf.solidShadedVtxBuffer.GetNumVertex(),
			    buf.solidShadedVtxBuffer,
			    buf.solidShadedNomBuffer,
			    buf.solidShadedColBuffer);
			YsGLSLEndUse3DRenderer(YsGLSLSharedVariColorShaded3DRenderer());
		}
		if(NULL!=YsGLSLSharedVariColor3DRenderer())
		{
			YsGLSLUse3DRenderer(YsGLSLSharedVariColor3DRenderer());

			YsGLSLDrawPrimitiveVtxColfv(
			    YsGLSLSharedVariColor3DRenderer(),
			    GL_TRIANGLES,
			    buf.solidUnshadedVtxBuffer.GetNumVertex(),
			    buf.solidUnshadedVtxBuffer,
			    buf.solidUnshadedColBuffer);

			YsGLSLDrawPrimitiveVtxColfv(
			    YsGLSLSharedVariColor3DRenderer(),
			    GL_TRIANGLES,
			    buf.backFaceVtxBuffer.GetNumVertex(),
			    buf.backFaceVtxBuffer,
			    buf.backFaceColBuffer);

			YsGLSLEndUse3DRenderer(YsGLSLSharedVariColor3DRenderer());
		}
		if(NULL!=YsGLSLSharedVariColorShaded3DRenderer())
		{
			YsGLSLUse3DRenderer(YsGLSLSharedVariColorShaded3DRenderer());
			YsGLSLDrawPrimitiveVtxNomColfv(
			    YsGLSLSharedVariColorShaded3DRenderer(),
			    GL_TRIANGLES,
			    buf.trspShadedVtxBuffer.GetNumVertex(),
			    buf.trspShadedVtxBuffer,
			    buf.trspShadedNomBuffer,
			    buf.trspShadedColBuffer);
			YsGLSLEndUse3DRenderer(YsGLSLSharedVariColorShaded3DRenderer());
		}
		if(NULL!=YsGLSLSharedVariColor3DRenderer())
		{
			YsGLSLUse3DRenderer(YsGLSLSharedVariColor3DRenderer());
			YsGLSLDrawPrimitiveVtxColfv(
			    YsGLSLSharedVariColor3DRenderer(),
			    GL_TRIANGLES,
			    buf.trspUnshadedVtxBuffer.GetNumVertex(),
			    buf.trspUnshadedVtxBuffer,
			    buf.trspUnshadedColBuffer);
			YsGLSLEndUse3DRenderer(YsGLSLSharedVariColor3DRenderer());
		}
		
		glDisable(GL_CULL_FACE);
	}

	if(YSTRUE==actuallyDrawOtherShell)
	{
		YsArray <const YsShellDnmContainer <YsShellExtEdit>::Node *,16> allNode;
		shlGrp.GetNodePointerAll(allNode);
		for(YSSIZE_T nodeIdx=0; nodeIdx<allNode.GetN(); ++nodeIdx)
		{
			if(slHd!=allNode[nodeIdx])
			{
				struct YsGLSL3DRenderer *renderer=YsGLSLSharedFlat3DRenderer();
				YsShellExtDrawingBuffer &buf=allNode[nodeIdx]->GetDrawingBuffer();
				YsGLSLUse3DRenderer(renderer);

				const GLfloat edgeCol[4]={0,0,0,0.2f};
				YsGLSLSet3DRendererUniformColorfv(renderer,edgeCol);
				YsGLSLDrawPrimitiveVtxfv(renderer,GL_LINES,buf.normalEdgePosBuffer.GetNumVertex(),buf.normalEdgePosBuffer);

				YsGLSLEndUse3DRenderer(renderer);
			}
		}
	}
}

void FsGui3DMainCanvas::DrawInDnmEditMode(void)
{
	auto renderer=YsGLSLSharedVariColorShaded3DRenderer();
	GLfloat curGLTfm[16];

	YsGLSLUse3DRenderer(renderer);
	YsGLSLGet3DRendererModelViewfv(curGLTfm,renderer);
	YsGLSLEndUse3DRenderer(renderer);

	YsMatrix4x4 curTfm(YSFALSE);
	curTfm.CreateFromOpenGlCompatibleMatrix(curGLTfm);

	glEnable(GL_CULL_FACE);

	auto rootArray=shlGrp.GetRootNodeArray();
	for(auto node : rootArray)
	{
		DrawInDnmEditModeRecursive(node,dnmState,curTfm);
	}

	YsGLSL3DRenderer *usedRenderer[4]=
	{
		YsGLSLSharedVariColorShaded3DRenderer(),
		YsGLSLSharedVariColor3DRenderer(),
		YsGLSLSharedVariColorShaded3DRenderer(),
		YsGLSLSharedVariColor3DRenderer(),
	};
	for(int i=0; i<4; ++i)
	{
		YsGLSLUse3DRenderer(usedRenderer[i]);
		YsGLSLSet3DRendererModelViewfv(usedRenderer[i],curGLTfm);
		YsGLSLEndUse3DRenderer(usedRenderer[i]);
	}

	glDisable(GL_CULL_FACE);
}

void FsGui3DMainCanvas::DrawInDnmEditModeRecursive(YsShellDnmContainer <YsShellExtEdit>::Node *node,const YsShellDnmContainer <YsShellExtEdit>::DnmState &state,const YsMatrix4x4 &curTfm)
{
	const YsMatrix4x4 tfm=curTfm*state.GetTransformation(node);

	GLfloat glTfm[16];
	tfm.GetOpenGlCompatibleMatrix(glTfm);

	for(auto childNode : node->children)
	{
		DrawInDnmEditModeRecursive(childNode,state,tfm);
	}

	{
		auto &buf=node->GetDrawingBuffer();

		if(NULL!=YsGLSLSharedVariColorShaded3DRenderer())
		{
			auto renderer=YsGLSLSharedVariColorShaded3DRenderer();
			YsGLSLUse3DRenderer(renderer);

			YsGLSLSet3DRendererModelViewfv(renderer,glTfm);
			YsGLSLDrawPrimitiveVtxNomColfv(
			    renderer,
			    GL_TRIANGLES,
			    buf.solidShadedVtxBuffer.GetNumVertex(),
			    buf.solidShadedVtxBuffer,
			    buf.solidShadedNomBuffer,
			    buf.solidShadedColBuffer);
			YsGLSLEndUse3DRenderer(renderer);
		}
		if(NULL!=YsGLSLSharedVariColor3DRenderer())
		{
			auto renderer=YsGLSLSharedVariColor3DRenderer();
			YsGLSLUse3DRenderer(renderer);

			YsGLSLSet3DRendererModelViewfv(renderer,glTfm);
			YsGLSLDrawPrimitiveVtxColfv(
			    renderer,
			    GL_TRIANGLES,
			    buf.solidUnshadedVtxBuffer.GetNumVertex(),
			    buf.solidUnshadedVtxBuffer,
			    buf.solidUnshadedColBuffer);

			YsGLSLEndUse3DRenderer(renderer);
		}
		if(NULL!=YsGLSLSharedVariColorShaded3DRenderer())
		{
			auto renderer=YsGLSLSharedVariColorShaded3DRenderer();
			YsGLSLUse3DRenderer(renderer);

			YsGLSLSet3DRendererModelViewfv(renderer,glTfm);
			YsGLSLDrawPrimitiveVtxNomColfv(
			    renderer,
			    GL_TRIANGLES,
			    buf.trspShadedVtxBuffer.GetNumVertex(),
			    buf.trspShadedVtxBuffer,
			    buf.trspShadedNomBuffer,
			    buf.trspShadedColBuffer);
			YsGLSLEndUse3DRenderer(renderer);
		}
		if(NULL!=YsGLSLSharedVariColor3DRenderer())
		{
			auto renderer=YsGLSLSharedVariColor3DRenderer();
			YsGLSLUse3DRenderer(renderer);

			YsGLSLSet3DRendererModelViewfv(renderer,glTfm);
			YsGLSLDrawPrimitiveVtxColfv(
			    renderer,
			    GL_TRIANGLES,
			    buf.trspUnshadedVtxBuffer.GetNumVertex(),
			    buf.trspUnshadedVtxBuffer,
			    buf.trspUnshadedColBuffer);
			YsGLSLEndUse3DRenderer(renderer);
		}
	}
}

void FsGui3DMainCanvas::RemakeDrawingBuffer(
    YsShellDnmContainer <YsShellExtEdit>::Node *slHd,
    unsigned int needRemakeDrawingBuffer)
{
	YsShellExtDrawingBuffer &buf=slHd->GetDrawingBuffer();

	buf.drawBackFaceInDifferentColor=drawBackFaceInDifferentColor;
	buf.backFaceColor=backFaceColor;

	if(0!=(needRemakeDrawingBuffer & NEED_REMAKE_DRAWING_VERTEX) && NULL!=slHd)
	{
		buf.RemakeNormalVertexBuffer(*((const YsShellExt *)slHd),normalVertexMarkerRadius,roundVertexMarkerRadius);
	}

	if(0!=(needRemakeDrawingBuffer & NEED_REMAKE_DRAWING_SELECTED_VERTEX))
	{
		YsArray <YsShellVertexHandle> selVtHd;
		slHd->GetSelectedVertex(selVtHd);
		buf.RemakeSelectedVertexBuffer(*((const YsShellExt *)slHd),selectedVertexMarkerRadius,selVtHd);
	}

	if(0!=(needRemakeDrawingBuffer & NEED_REMAKE_DRAWING_POLYGON))
	{
		buf.RemakePolygonBuffer(*((const YsShellExt *)slHd),polygonShrinkRatioForPicking);
	}

	if(0!=(needRemakeDrawingBuffer & NEED_REMAKE_DRAWING_SELECTED_POLYGON))
	{
		YsArray <YsShellPolygonHandle> selPlHd;
		slHd->GetSelectedPolygon(selPlHd);
		buf.RemakeSelectedPolygonBuffer(*((const YsShellExt *)slHd),polygonShrinkRatioForPicking,selPlHd);
	}

	if(0!=(needRemakeDrawingBuffer & (NEED_REMAKE_DRAWING_CONSTEDGE|NEED_REMAKE_DRAWING_SELECTED_CONSTEDGE)))
	{
		YsArray <YsShellExt::ConstEdgeHandle> selCeHd;
		slHd->GetSelectedConstEdge(selCeHd);
		buf.RemakeConstEdgeBuffer(*((const YsShellExt *)slHd),selCeHd);
		buf.RemakeSelectedConstEdgeBuffer(*((const YsShellExt *)slHd),selCeHd);
	}

	if(0!=(needRemakeDrawingBuffer & (NEED_REMAKE_DRAWING_SELECTED_FACEGROUP)))
	{
		YsArray <YsShellExt::FaceGroupHandle> selFgHd;
		slHd->GetSelectedFaceGroup(selFgHd);
		buf.RemakeSelectedFaceGroupBuffer(*((const YsShellExt *)slHd),selFgHd);
	}
}

