/* ////////////////////////////////////////////////////////////

  File Name: selectmenu.cpp
  Copyright (c) 2014 Soji Yamakawa.  All rights reserved.
  http://www.ysflight.com

//////////////////////////////////////////////////////////// */

#include "fsgui3dapp.h"

#include <ysgl.h>
#include <ysshellext_trackingutil.h>


void FsGui3DMainCanvas::Select_UnselectAll(void *appPtr,FsGuiPopUpMenu *,FsGuiPopUpMenuItem *)
{
	FsGui3DMainCanvas *canvas=(FsGui3DMainCanvas *)appPtr;
	if(NULL!=canvas->slHd)
	{
		canvas->slHd->SelectVertex(0,NULL);
		canvas->slHd->SelectPolygon(0,NULL);
		canvas->slHd->SelectConstEdge(0,NULL);
		canvas->slHd->SelectFaceGroup(0,NULL);

		canvas->needRemakeDrawingBuffer|=NEED_REMAKE_DRAWING_SELECTED_VERTEX|NEED_REMAKE_DRAWING_SELECTED_POLYGON|
		                                 NEED_REMAKE_DRAWING_SELECTED_CONSTEDGE|NEED_REMAKE_DRAWING_SELECTED_FACEGROUP;
		canvas->SetNeedRedraw(YSTRUE);
	}
}

void FsGui3DMainCanvas::Select_PickVertex(void *appPtr,FsGuiPopUpMenu *,FsGuiPopUpMenuItem *)
{
	FsGui3DMainCanvas *canvas=(FsGui3DMainCanvas *)appPtr;
	Edit_ClearUIMode(appPtr,NULL,NULL);
	canvas->undoRedoRequireToClearUIMode=YSFALSE;
	canvas->deletionRequireToClearUIMode=YSFALSE;
	canvas->LButtonCallBack=Select_PickVertex_LButtonCallBack;
	canvas->LButtonBoxCallBack=Select_PickVertex_LButtonBoxCallBack;
	canvas->BackSpaceKeyCallBack=Select_PickVertex_BackSpaceKeyCallBack;

	canvas->SetNeedRedraw(YSTRUE);
}

YSRESULT FsGui3DMainCanvas::Select_PickVertex_LButtonCallBack(FsGui3DMainCanvas &canvas,YSBOOL,YSBOOL,YSBOOL,int mx,int my)
{
	if(NULL!=canvas.slHd)
	{
		YsArray <YsShellVertexHandle> selVtHd;
		canvas.slHd->GetSelectedVertex(selVtHd);

		if(0==FsGetKeyState(FSKEY_CTRL))
		{
			YsShellVertexHandle vtHd[1]={canvas.PickedVtHd(mx,my,selVtHd)};

			if(NULL!=vtHd[0])
			{
				canvas.slHd->AddSelectedVertex(1,vtHd);
				canvas.needRemakeDrawingBuffer|=NEED_REMAKE_DRAWING_SELECTED_VERTEX;
				canvas.SetNeedRedraw(YSTRUE);
				return YSOK;
			}
		}
		else
		{
			YsShellVertexHandle vtHd=canvas.PickedVtHdAmongSelection(mx,my);  // Selection can be already cached in the vertex buffer.

			if(NULL!=vtHd)
			{
				for(YSSIZE_T idx=selVtHd.GetN()-1; 0<=idx; --idx)
				{
					if(selVtHd[idx]==vtHd)
					{
						selVtHd.Delete(idx);
					}
				}

				canvas.slHd->SelectVertex(selVtHd);
				canvas.needRemakeDrawingBuffer|=NEED_REMAKE_DRAWING_SELECTED_VERTEX;
				canvas.SetNeedRedraw(YSTRUE);
				return YSOK;
			}
		}
	}
	return YSERR;
}

/*static*/ YSRESULT FsGui3DMainCanvas::Select_PickVertex_LButtonBoxCallBack(FsGui3DMainCanvas &canvas,int xMin,int yMin,int xMax,int yMax)
{
	if(NULL!=canvas.slHd)
	{
		YsArray <YsShellVertexHandle> selVtHd;
		canvas.slHd->GetSelectedVertex(selVtHd);

		if(0==FsGetKeyState(FSKEY_CTRL))
		{
			YsArray <YsPair <YsShellVertexHandle,double> > vtHdDepthPair=canvas.BoxedVtHd(xMin,yMin,xMax,yMax,selVtHd);
			if(0<vtHdDepthPair.GetN())
			{
				for(auto pair : vtHdDepthPair)
				{
					selVtHd.Append(pair.a);
				}
				canvas.slHd->SelectVertex(selVtHd);
				canvas.needRemakeDrawingBuffer|=NEED_REMAKE_DRAWING_SELECTED_VERTEX;
				canvas.SetNeedRedraw(YSTRUE);
			}
		}
		else
		{
			YsArray <YsPair <YsShellVertexHandle,double> > vtHdDepthPair=canvas.BoxedVtHdAmongSelection(xMin,yMin,xMax,yMax);
			if(0<vtHdDepthPair.GetN())
			{
				YsKeyStore boxed;
				for(auto pair : vtHdDepthPair)
				{
					boxed.AddKey(canvas.slHd->GetSearchKey(pair.a));
				}
				YsArray <YsShellVertexHandle> newSelVtHd;
				for(YSSIZE_T idx=selVtHd.GetN()-1; 0<=idx; --idx)
				{
					if(YSTRUE!=boxed.IsIncluded(canvas.slHd->GetSearchKey(selVtHd[idx])))
					{
						newSelVtHd.Append(selVtHd[idx]);
					}
				}
				canvas.slHd->SelectVertex(newSelVtHd);
				canvas.needRemakeDrawingBuffer|=NEED_REMAKE_DRAWING_SELECTED_VERTEX;
				canvas.SetNeedRedraw(YSTRUE);
			}
		}
	}

	return YSERR;
}

/*static*/ YSRESULT FsGui3DMainCanvas::Select_PickVertex_BackSpaceKeyCallBack(FsGui3DMainCanvas &canvas)
{
	if(NULL!=canvas.slHd)
	{
		auto selVtHd=canvas.slHd->GetSelectedVertex();
		if(0<selVtHd.GetN())
		{
			selVtHd.DeleteLast();
			canvas.slHd->SelectVertex(selVtHd);
			canvas.needRemakeDrawingBuffer|=NEED_REMAKE_DRAWING_SELECTED_VERTEX;
			canvas.SetNeedRedraw(YSTRUE);
			return YSOK;
		}
	}
	return YSERR;
}

void FsGui3DMainCanvas::Select_PickPolygon(void *appPtr,FsGuiPopUpMenu *,FsGuiPopUpMenuItem *)
{
	FsGui3DMainCanvas *canvas=(FsGui3DMainCanvas *)appPtr;
	Edit_ClearUIMode(appPtr,NULL,NULL);
	canvas->undoRedoRequireToClearUIMode=YSFALSE;
	canvas->deletionRequireToClearUIMode=YSFALSE;
	canvas->LButtonCallBack=Select_PickPolygon_LButtonCallBack;
	canvas->LButtonBoxCallBack=Select_PickPolygon_LButtonBoxCallBack;
	canvas->BackSpaceKeyCallBack=Select_PickPolygon_BackSpaceKeyCallBack;
	canvas->SetNeedRedraw(YSTRUE);
}

YSRESULT FsGui3DMainCanvas::Select_PickPolygon_LButtonCallBack(FsGui3DMainCanvas &canvas,YSBOOL,YSBOOL,YSBOOL,int mx,int my)
{
	if(NULL!=canvas.slHd)
	{
		YsArray <YsShellPolygonHandle> selPlHd;
		canvas.slHd->GetSelectedPolygon(selPlHd);

		if(0==FsGetKeyState(FSKEY_CTRL))
		{
			YsShellPolygonHandle plHd[1]={canvas.PickedPlHd(mx,my,selPlHd)};
			if(NULL!=plHd[0])
			{
				canvas.slHd->AddSelectedPolygon(1,plHd);
				canvas.needRemakeDrawingBuffer|=NEED_REMAKE_DRAWING_SELECTED_POLYGON;
				canvas.SetNeedRedraw(YSTRUE);
				return YSOK;
			}
		}
		else 
		{
			YsShellPolygonHandle plHd=canvas.PickedPlHdAmongSelection(mx,my);
			if(NULL!=plHd)
			{
				for(YSSIZE_T idx=selPlHd.GetN()-1; 0<=idx; --idx)
				{
					if(selPlHd[idx]==plHd)
					{
						selPlHd.Delete(idx);
					}
				}
				canvas.slHd->SelectPolygon(selPlHd);
				canvas.needRemakeDrawingBuffer|=NEED_REMAKE_DRAWING_SELECTED_POLYGON;
				canvas.SetNeedRedraw(YSTRUE);
				return YSOK;
			}
		}
	}
	return YSERR;
}

/*static*/ YSRESULT FsGui3DMainCanvas::Select_PickPolygon_LButtonBoxCallBack(FsGui3DMainCanvas &canvas,int xMin,int yMin,int xMax,int yMax)
{
	if(NULL!=canvas.slHd)
	{
		YsArray <YsShellPolygonHandle> selPlHd;
		canvas.slHd->GetSelectedPolygon(selPlHd);

		if(0==FsGetKeyState(FSKEY_CTRL))
		{
			YsArray <YsPair <YsShellPolygonHandle,double> > plHdDepthPairArray=canvas.BoxedPlHd(xMin,yMin,xMax,yMax,selPlHd);
			if(0<plHdDepthPairArray.GetN())
			{
				for(auto pair : plHdDepthPairArray)
				{
					selPlHd.Append(pair.a);
				}

				canvas.slHd->SelectPolygon(selPlHd);
				canvas.needRemakeDrawingBuffer|=NEED_REMAKE_DRAWING_SELECTED_POLYGON;
				canvas.SetNeedRedraw(YSTRUE);
				return YSOK;
			}
		}
		else 
		{
			YsArray <YsPair <YsShellPolygonHandle,double> > plHdDepthPairArray=canvas.BoxedPlHdAmongSelection(xMin,yMin,xMax,yMax);
			if(0<plHdDepthPairArray.GetN())
			{
				YsShellPolygonStore boxed(*(const YsShell *)canvas.slHd);
				for(auto pair : plHdDepthPairArray)
				{
					boxed.AddPolygon(pair.a);
				}

				YsArray <YsShellPolygonHandle> newSelPlHd;
				for(auto plHd : selPlHd)
				{
					if(YSTRUE!=boxed.IsIncluded(plHd))
					{
						newSelPlHd.Append(plHd);
					}
				}

				canvas.slHd->SelectPolygon(newSelPlHd);
				canvas.needRemakeDrawingBuffer|=NEED_REMAKE_DRAWING_SELECTED_POLYGON;
				canvas.SetNeedRedraw(YSTRUE);
				return YSOK;
			}
		}
	}
	return YSERR;
}

/*static*/ YSRESULT FsGui3DMainCanvas::Select_PickPolygon_BackSpaceKeyCallBack(FsGui3DMainCanvas &canvas)
{
	if(NULL!=canvas.slHd)
	{
		auto selPlHd=canvas.slHd->GetSelectedPolygon();
		if(0<selPlHd.GetN())
		{
			selPlHd.DeleteLast();
			canvas.slHd->SelectPolygon(selPlHd);
			canvas.needRemakeDrawingBuffer|=NEED_REMAKE_DRAWING_SELECTED_POLYGON;
			canvas.SetNeedRedraw(YSTRUE);
			return YSOK;
		}
	}
	return YSERR;
}

void FsGui3DMainCanvas::Select_PickConstEdge(void *appPtr,FsGuiPopUpMenu *,FsGuiPopUpMenuItem *)
{
	FsGui3DMainCanvas *canvas=(FsGui3DMainCanvas *)appPtr;
	Edit_ClearUIMode(appPtr,NULL,NULL);
	canvas->undoRedoRequireToClearUIMode=YSFALSE;
	canvas->deletionRequireToClearUIMode=YSFALSE;
	canvas->LButtonCallBack=Select_PickConstEdge_LButtonCallBack;
	canvas->LButtonBoxCallBack=Select_PickConstEdge_LButtonBoxCallBack;
	canvas->BackSpaceKeyCallBack=Select_PickConstEdge_BackSpaceKeyCallBack;
	canvas->SetNeedRedraw(YSTRUE);
}

YSRESULT FsGui3DMainCanvas::Select_PickConstEdge_LButtonCallBack(FsGui3DMainCanvas &canvas,YSBOOL,YSBOOL,YSBOOL,int mx,int my)
{
	if(NULL!=canvas.slHd)
	{
		YsArray <YsShellExt::ConstEdgeHandle> selCeHd;
		canvas.slHd->GetSelectedConstEdge(selCeHd);

		if(0==FsGetKeyState(FSKEY_CTRL))
		{
			YsShellExt::ConstEdgeHandle ceHd[1]={canvas.PickedCeHd(mx,my,selCeHd)};
			if(NULL!=ceHd[0])
			{
				canvas.slHd->AddSelectedConstEdge(1,ceHd);
				canvas.needRemakeDrawingBuffer|=NEED_REMAKE_DRAWING_SELECTED_CONSTEDGE;
				canvas.SetNeedRedraw(YSTRUE);
				return YSOK;
			}
		}
		else
		{
			YsShellExt::ConstEdgeHandle ceHd=canvas.PickedCeHdAmongSelection(mx,my);
			if(NULL!=ceHd)
			{
				for(YSSIZE_T idx=selCeHd.GetN()-1; 0<=idx; --idx)
				{
					if(ceHd==selCeHd[idx])
					{
						selCeHd.Delete(idx);
					}
				}
				canvas.slHd->SelectConstEdge(selCeHd);
				canvas.needRemakeDrawingBuffer|=NEED_REMAKE_DRAWING_SELECTED_CONSTEDGE;
				canvas.SetNeedRedraw(YSTRUE);
				return YSOK;
			}
		}
	}
	return YSERR;
}

YSRESULT FsGui3DMainCanvas::Select_PickConstEdge_LButtonBoxCallBack(FsGui3DMainCanvas &canvas,int x0,int y0,int x1,int y1)
{
	if(NULL!=canvas.slHd)
	{
		YsArray <YsShellExt::ConstEdgeHandle> selCeHd;
		canvas.slHd->GetSelectedConstEdge(selCeHd);

		if(0==FsGetKeyState(FSKEY_CTRL))
		{
			YsArray <YsPair <YsShellExt::ConstEdgeHandle,double> > boxed=canvas.BoxedCeHd(x0,y0,x1,y1,selCeHd);
			if(0<boxed.GetN())
			{
				for(auto pair : boxed)
				{
					selCeHd.Append(pair.a);
				}
				canvas.slHd->SelectConstEdge(selCeHd);
				canvas.needRemakeDrawingBuffer|=NEED_REMAKE_DRAWING_SELECTED_CONSTEDGE;
				canvas.SetNeedRedraw(YSTRUE);
				return YSOK;
			}
		}
		else
		{
			YsArray <YsPair <YsShellExt::ConstEdgeHandle,double> > boxed=canvas.BoxedCeHdAmongSelection(x0,y0,x1,y1);
			if(0<boxed.GetN())
			{
				YsKeyStore boxedCeKey;
				for(auto pair : boxed)
				{
					boxedCeKey.AddKey(canvas.slHd->GetSearchKey(pair.a));
				}
				YsArray <YsShellExt::ConstEdgeHandle> newSelCeHd;
				for(auto ceHd : selCeHd)
				{
					if(YSTRUE!=boxedCeKey.IsIncluded(canvas.slHd->GetSearchKey(ceHd)))
					{
						newSelCeHd.Append(ceHd);
					}
				}
				canvas.slHd->SelectConstEdge(newSelCeHd);
				canvas.needRemakeDrawingBuffer|=NEED_REMAKE_DRAWING_SELECTED_CONSTEDGE;
				canvas.SetNeedRedraw(YSTRUE);
				return YSOK;
			}
		}
	}
	return YSERR;
}

/*static*/ YSRESULT FsGui3DMainCanvas::Select_PickConstEdge_BackSpaceKeyCallBack(FsGui3DMainCanvas &canvas)
{
	if(NULL!=canvas.slHd)
	{
		auto selCeHd=canvas.slHd->GetSelectedConstEdge();
		if(0<selCeHd.GetN())
		{
			selCeHd.DeleteLast();
			canvas.slHd->SelectConstEdge(selCeHd);
			canvas.needRemakeDrawingBuffer|=NEED_REMAKE_DRAWING_SELECTED_CONSTEDGE;
			canvas.SetNeedRedraw(YSTRUE);
			return YSOK;
		}
	}
	return YSERR;
}

void FsGui3DMainCanvas::Select_PickFaceGroup(void *appPtr,FsGuiPopUpMenu *,FsGuiPopUpMenuItem *)
{
	FsGui3DMainCanvas *canvas=(FsGui3DMainCanvas *)appPtr;
	Edit_ClearUIMode(appPtr,NULL,NULL);
	canvas->undoRedoRequireToClearUIMode=YSFALSE;
	canvas->deletionRequireToClearUIMode=YSFALSE;
	canvas->LButtonCallBack=Select_PickFaceGroup_LButtonCallBack;
	canvas->BackSpaceKeyCallBack=Select_PickFaceGroup_BackSpaceKeyCallBack;
	canvas->SetNeedRedraw(YSTRUE);
}

YSRESULT FsGui3DMainCanvas::Select_PickFaceGroup_LButtonCallBack(FsGui3DMainCanvas &canvas,YSBOOL,YSBOOL,YSBOOL,int mx,int my)
{
	if(NULL!=canvas.slHd)
	{
		YsArray <YsShellExt::FaceGroupHandle> selFgHd;
		canvas.slHd->GetSelectedFaceGroup(selFgHd);

		if(0==FsGetKeyState(FSKEY_CTRL))
		{
			YsShellExt::FaceGroupHandle pickedFgHd[1]={canvas.PickedFgHd(mx,my,selFgHd)};
			if(NULL!=pickedFgHd[0])
			{
				canvas.slHd->AddSelectedFaceGroup(1,pickedFgHd);
				canvas.needRemakeDrawingBuffer|=NEED_REMAKE_DRAWING_SELECTED_FACEGROUP;
				canvas.SetNeedRedraw(YSTRUE);
			}
		}
		else 
		{
			auto pickedFgHd=canvas.PickedFgHdAmongSelection(mx,my);
			if(NULL!=pickedFgHd)
			{
				for(YSSIZE_T idx=selFgHd.GetN()-1; 0<=idx; --idx)
				{
					if(selFgHd[idx]==pickedFgHd)
					{
						selFgHd.Delete(idx);
					}
				}
				canvas.slHd->SelectFaceGroup(selFgHd);
				canvas.needRemakeDrawingBuffer|=NEED_REMAKE_DRAWING_SELECTED_FACEGROUP;
				canvas.SetNeedRedraw(YSTRUE);
				return YSOK;
			}
		}

		return YSOK;
	}
	return YSERR;
}

/*static*/ YSRESULT FsGui3DMainCanvas::Select_PickFaceGroup_BackSpaceKeyCallBack(FsGui3DMainCanvas &canvas)
{
	if(NULL!=canvas.slHd)
	{
		auto selFgHd=canvas.slHd->GetSelectedFaceGroup();
		if(0<selFgHd.GetN())
		{
			selFgHd.DeleteLast();
			canvas.slHd->SelectFaceGroup(selFgHd);
			canvas.needRemakeDrawingBuffer|=NEED_REMAKE_DRAWING_SELECTED_FACEGROUP;
			canvas.SetNeedRedraw(YSTRUE);
			return YSOK;
		}
	}
	return YSERR;
}

/*static*/ void FsGui3DMainCanvas::Select_UnselectVertex(void *appPtr,FsGuiPopUpMenu *,FsGuiPopUpMenuItem *)
{
	FsGui3DMainCanvas *canvas=(FsGui3DMainCanvas *)appPtr;
	if(NULL!=canvas->slHd)
	{
		canvas->slHd->SelectVertex(0,NULL);
		canvas->needRemakeDrawingBuffer|=NEED_REMAKE_DRAWING_SELECTED_VERTEX;
		canvas->SetNeedRedraw(YSTRUE);
	}
}

/*static*/ void FsGui3DMainCanvas::Select_AllVertex(void *appPtr,FsGuiPopUpMenu *,FsGuiPopUpMenuItem *)
{
	FsGui3DMainCanvas *canvas=(FsGui3DMainCanvas *)appPtr;
	if(NULL!=canvas->slHd)
	{
		YsArray <YsShellVertexHandle> allVtHd;
		for(YsShellVertexHandle vtHd=NULL; YSOK==canvas->slHd->MoveToNextVertex(vtHd); )
		{
			allVtHd.Append(vtHd);
		}
		canvas->slHd->SelectVertex(allVtHd);
		canvas->needRemakeDrawingBuffer|=NEED_REMAKE_DRAWING_SELECTED_VERTEX;
		canvas->SetNeedRedraw(YSTRUE);
	}
}


/*static*/ void FsGui3DMainCanvas::Select_InvertVertex(void *appPtr,FsGuiPopUpMenu *,FsGuiPopUpMenuItem *)
{
	FsGui3DMainCanvas *canvas=(FsGui3DMainCanvas *)appPtr;
	if(NULL!=canvas->slHd)
	{
		YsArray <YsShellVertexHandle> selVtHd;
		canvas->slHd->GetSelectedVertex(selVtHd);

		YsShellVertexStore selVtHash(*(const YsShell *)(canvas->slHd),selVtHd);

		YsArray <YsShellVertexHandle> allVtHd;
		for(YsShellVertexHandle vtHd=NULL; YSOK==canvas->slHd->MoveToNextVertex(vtHd); )
		{
			if(YSTRUE!=selVtHash.IsIncluded(vtHd))
			{
				allVtHd.Append(vtHd);
			}
		}
		canvas->slHd->SelectVertex(allVtHd);
		canvas->needRemakeDrawingBuffer|=NEED_REMAKE_DRAWING_SELECTED_VERTEX;
		canvas->SetNeedRedraw(YSTRUE);
	}
}


/*static*/ void FsGui3DMainCanvas::Select_VertexOfPolygon(void *appPtr,FsGuiPopUpMenu *,FsGuiPopUpMenuItem *)
{
	FsGui3DMainCanvas *canvas=(FsGui3DMainCanvas *)appPtr;
	if(NULL!=canvas->slHd)
	{
		YsArray <YsShellPolygonHandle> selPlHd;
		canvas->slHd->GetSelectedPolygon(selPlHd);

		YsShellVertexStore visited(*(const YsShell *)canvas->slHd);

		YsArray <YsShellVertexHandle> selVtHd;
		for(YSSIZE_T plIdx=0; plIdx<selPlHd.GetN(); ++plIdx)
		{
			int nPlVt;
			const YsShellVertexHandle *plVtHd;
			canvas->slHd->GetVertexListOfPolygon(nPlVt,plVtHd,selPlHd[plIdx]);

			for(YSSIZE_T vtIdx=0; vtIdx<nPlVt; ++vtIdx)
			{
				if(YSTRUE!=visited.IsIncluded(plVtHd[vtIdx]))
				{
					selVtHd.Append(plVtHd[vtIdx]);
					visited.AddVertex(plVtHd[vtIdx]);
				}
			}
		}

		canvas->slHd->SelectVertex(selVtHd);
		canvas->needRemakeDrawingBuffer|=NEED_REMAKE_DRAWING_SELECTED_VERTEX;
		canvas->SetNeedRedraw(YSTRUE);
	}
}


/*static*/ void FsGui3DMainCanvas::Select_VertexOfConstEdge(void *appPtr,FsGuiPopUpMenu *,FsGuiPopUpMenuItem *)
{
	FsGui3DMainCanvas *canvas=(FsGui3DMainCanvas *)appPtr;
	if(NULL!=canvas->slHd)
	{
		YsArray <YsShellExt::ConstEdgeHandle> selCeHd;
		canvas->slHd->GetSelectedConstEdge(selCeHd);

		YsShellVertexStore visited(*(const YsShell *)canvas->slHd);

		YsArray <YsShellVertexHandle> selVtHd;
		for(YSSIZE_T ceIdx=0; ceIdx<selCeHd.GetN(); ++ceIdx)
		{
			YSSIZE_T nCeVt;
			const YsShellVertexHandle *ceVtHd;
			YSBOOL isLoop;
			canvas->slHd->GetConstEdge(nCeVt,ceVtHd,isLoop,selCeHd[ceIdx]);

			for(YSSIZE_T vtIdx=0; vtIdx<nCeVt; ++vtIdx)
			{
				if(YSTRUE!=visited.IsIncluded(ceVtHd[vtIdx]))
				{
					selVtHd.Append(ceVtHd[vtIdx]);
					visited.AddVertex(ceVtHd[vtIdx]);
				}
			}
		}

		canvas->slHd->SelectVertex(selVtHd);
		canvas->needRemakeDrawingBuffer|=NEED_REMAKE_DRAWING_SELECTED_VERTEX;
		canvas->SetNeedRedraw(YSTRUE);
	}
}

/*static*/ void FsGui3DMainCanvas::Select_VertexAlongBoundaryOrConstEdge(void *appPtr,FsGuiPopUpMenu *,FsGuiPopUpMenuItem *)
{
	FsGui3DMainCanvas &canvas=*(FsGui3DMainCanvas *)appPtr;
	if(NULL!=canvas.slHd)
	{
		auto selVtHd=canvas.slHd->GetSelectedVertex();
		if(0<selVtHd.GetN())
		{
			YsArray <YsShellVertexHandle> connVtHd;
			canvas.slHd->GetConnectedVertex(connVtHd,selVtHd.Last());

			YsArray <YsShellVertexHandle> candidate;
			for(auto vtHd : connVtHd)
			{
				if(1==canvas.slHd->GetNumPolygonUsingEdge(selVtHd.Last(),vtHd) &&
				   YSTRUE!=selVtHd.IsIncluded(vtHd))
				{
					candidate.Append(vtHd);
				}
			}

			if(1==candidate.GetN())
			{
				selVtHd.Append(candidate[0]);
				canvas.slHd->SelectVertex(selVtHd);
			}
			else if(2<=candidate.GetN())
			{
				if(0==FsGetKeyState(FSKEY_SHIFT))
				{
					selVtHd.Append(candidate[0]);
					canvas.slHd->SelectVertex(selVtHd);
				}
				else
				{
					selVtHd.Append(candidate[1]);
					canvas.slHd->SelectVertex(selVtHd);
				}
			}
			else
			{
				auto candidate=canvas.slHd->GetConstEdgeConnectedVertex(selVtHd.Last());
				for(auto idx=candidate.GetN()-1; 0<=idx; --idx)
				{
					if(YSTRUE==selVtHd.IsIncluded(candidate[idx]))
					{
						candidate.Delete(idx);
					}
				}
				if(1==candidate.GetN())
				{
					selVtHd.Append(candidate[0]);
					canvas.slHd->SelectVertex(selVtHd);
				}
				else if(2<=candidate.GetN())
				{
					if(0==FsGetKeyState(FSKEY_SHIFT))
					{
						selVtHd.Append(candidate[0]);
						canvas.slHd->SelectVertex(selVtHd);
					}
					else
					{
						selVtHd.Append(candidate[1]);
						canvas.slHd->SelectVertex(selVtHd);
					}
				}
			}

			canvas.needRemakeDrawingBuffer|=NEED_REMAKE_DRAWING_SELECTED_VERTEX;
			canvas.SetNeedRedraw(YSTRUE);
		}
	}
}

/*static*/ void FsGui3DMainCanvas::Select_UnselectPolygon(void *appPtr,FsGuiPopUpMenu *,FsGuiPopUpMenuItem *)
{
	FsGui3DMainCanvas *canvas=(FsGui3DMainCanvas *)appPtr;
	if(NULL!=canvas->slHd)
	{
		canvas->slHd->SelectPolygon(0,NULL);
		canvas->needRemakeDrawingBuffer|=NEED_REMAKE_DRAWING_SELECTED_POLYGON;
		canvas->SetNeedRedraw(YSTRUE);
	}
}

/*static*/ void FsGui3DMainCanvas::Select_AllPolygon(void *appPtr,FsGuiPopUpMenu *,FsGuiPopUpMenuItem *)
{
	FsGui3DMainCanvas *canvas=(FsGui3DMainCanvas *)appPtr;
	if(NULL!=canvas->slHd)
	{
		YsArray <YsShellPolygonHandle> allPlHd;
		for(YsShellPolygonHandle plHd=NULL; YSOK==canvas->slHd->MoveToNextPolygon(plHd); )
		{
			allPlHd.Append(plHd);
		}

		canvas->slHd->SelectPolygon(allPlHd);
		canvas->needRemakeDrawingBuffer|=NEED_REMAKE_DRAWING_SELECTED_POLYGON;
		canvas->SetNeedRedraw(YSTRUE);
	}
}


/*static*/ void FsGui3DMainCanvas::Select_InvertPolygon(void *appPtr,FsGuiPopUpMenu *,FsGuiPopUpMenuItem *)
{
	FsGui3DMainCanvas *canvas=(FsGui3DMainCanvas *)appPtr;
	if(NULL!=canvas->slHd)
	{
		YsArray <YsShellPolygonHandle> selPlHd;
		canvas->slHd->GetSelectedPolygon(selPlHd);

		YsShellPolygonStore selPlHash(*(const YsShell *)canvas->slHd,selPlHd);

		YsArray <YsShellPolygonHandle> allPlHd;
		for(YsShellPolygonHandle plHd=NULL; NULL!=canvas->slHd->MoveToNextPolygon(plHd); )
		{
			if(YSTRUE!=selPlHash.IsIncluded(plHd))
			{
				allPlHd.Append(plHd);
			}
		}

		canvas->slHd->SelectPolygon(allPlHd);
		canvas->needRemakeDrawingBuffer|=NEED_REMAKE_DRAWING_SELECTED_POLYGON;
		canvas->SetNeedRedraw(YSTRUE);
	}
}


/*static*/ void FsGui3DMainCanvas::Select_PolygonOfFaceGroup(void *appPtr,FsGuiPopUpMenu *,FsGuiPopUpMenuItem *)
{
	FsGui3DMainCanvas *canvas=(FsGui3DMainCanvas *)appPtr;
	if(NULL!=canvas->slHd)
	{
		YsArray <YsShellExt::FaceGroupHandle> selFgHd;
		canvas->slHd->GetSelectedFaceGroup(selFgHd);

		YsShellPolygonStore visited(*(const YsShell *)(canvas->slHd));

		YsArray <YsShellPolygonHandle> allPlHd;
		for(YSSIZE_T fgIdx=0; fgIdx<selFgHd.GetN(); ++fgIdx)
		{
			YsArray <YsShellPolygonHandle> fgPlHd;
			canvas->slHd->GetFaceGroup(fgPlHd,selFgHd[fgIdx]);

			for(YSSIZE_T plIdx=0; plIdx<fgPlHd.GetN(); ++plIdx)
			{
				if(YSTRUE!=visited.IsIncluded(fgPlHd[plIdx]))
				{
					allPlHd.Append(fgPlHd[plIdx]);
					visited.AddPolygon(fgPlHd[plIdx]);
				}
			}
		}

		canvas->slHd->SelectPolygon(allPlHd);
		canvas->needRemakeDrawingBuffer|=NEED_REMAKE_DRAWING_SELECTED_POLYGON;
		canvas->SetNeedRedraw(YSTRUE);
	}
}

/*static*/ void FsGui3DMainCanvas::Select_CurrentShellByPicking(void *appPtr,FsGuiPopUpMenu *,FsGuiPopUpMenuItem *)
{
	FsGui3DMainCanvas *canvas=(FsGui3DMainCanvas *)appPtr;
	Edit_ClearUIMode(appPtr,NULL,NULL);
	canvas->undoRedoRequireToClearUIMode=YSFALSE;
	canvas->deletionRequireToClearUIMode=YSFALSE;
	canvas->LButtonCallBack=Select_CurrentShellByPicking_LButtonCallBack;
	canvas->SetNeedRedraw(YSTRUE);
}

/*static*/ YSRESULT FsGui3DMainCanvas::Select_CurrentShellByPicking_LButtonCallBack(FsGui3DMainCanvas &canvas,YSBOOL,YSBOOL,YSBOOL,int mx,int my)
{
	const YsShellDnmContainer <YsShellExtEdit>::Node *excludeSlHd[1]={canvas.slHd};
	YsShellDnmContainer <YsShellExtEdit>::Node *pickedSlHd=canvas.PickedSlHd(mx,my,1,excludeSlHd);
	if(NULL!=pickedSlHd)
	{
		canvas.Edit_ClearUIMode();
		canvas.slHd=pickedSlHd;
		canvas.needRemakeDrawingBuffer|=NEED_REMAKE_DRAWING_ALL;
		canvas.needRemakeDrawingBuffer_for_OtherShell|=NEED_REMAKE_DRAWING_ALL;
		canvas.SetNeedRedraw(YSTRUE);
		return YSOK;
	}
	return YSERR;
}

/*static*/ void FsGui3DMainCanvas::Select_NextShell(void *appPtr,FsGuiPopUpMenu *,FsGuiPopUpMenuItem *)
{
	FsGui3DMainCanvas *canvas=(FsGui3DMainCanvas *)appPtr;

	YsArray <YsShellDnmContainer <YsShellExtEdit>::Node *> allNode;
	canvas->shlGrp.GetNodePointerAll(allNode);
	if(0<allNode.GetN())
	{
		if(NULL!=canvas->slHd)
		{
			for(YSSIZE_T idx=0; idx<allNode.GetN(); ++idx)
			{
				if(canvas->slHd==allNode[idx])
				{
					canvas->slHd=allNode.GetCyclic(idx+1);
					canvas->needRemakeDrawingBuffer|=NEED_REMAKE_DRAWING_ALL;
					canvas->needRemakeDrawingBuffer_for_OtherShell|=NEED_REMAKE_DRAWING_ALL;
					canvas->SetNeedRedraw(YSTRUE);
					return;
				}
			}
		}
		else
		{
			canvas->slHd=allNode[0];
			canvas->needRemakeDrawingBuffer|=NEED_REMAKE_DRAWING_ALL;
			canvas->needRemakeDrawingBuffer_for_OtherShell|=NEED_REMAKE_DRAWING_ALL;
			canvas->SetNeedRedraw(YSTRUE);
		}
	}
}

/*static*/ void FsGui3DMainCanvas::Select_PrevShell(void *appPtr,FsGuiPopUpMenu *,FsGuiPopUpMenuItem *)
{
	FsGui3DMainCanvas *canvas=(FsGui3DMainCanvas *)appPtr;

	YsArray <YsShellDnmContainer <YsShellExtEdit>::Node *> allNode;
	canvas->shlGrp.GetNodePointerAll(allNode);
	if(0<allNode.GetN())
	{
		if(NULL!=canvas->slHd)
		{
			for(YSSIZE_T idx=0; idx<allNode.GetN(); ++idx)
			{
				if(canvas->slHd==allNode[idx])
				{
					canvas->slHd=allNode.GetCyclic(idx-1);
					canvas->needRemakeDrawingBuffer|=NEED_REMAKE_DRAWING_ALL;
					canvas->needRemakeDrawingBuffer_for_OtherShell|=NEED_REMAKE_DRAWING_ALL;
					canvas->SetNeedRedraw(YSTRUE);
					return;
				}
			}
		}
		else
		{
			canvas->slHd=allNode[0];
			canvas->needRemakeDrawingBuffer|=NEED_REMAKE_DRAWING_ALL;
			canvas->needRemakeDrawingBuffer_for_OtherShell|=NEED_REMAKE_DRAWING_ALL;
			canvas->SetNeedRedraw(YSTRUE);
		}
	}
}

/*static*/ void FsGui3DMainCanvas::Select_PolygonConnected(void *appPtr,FsGuiPopUpMenu *,FsGuiPopUpMenuItem *)
{
	auto &canvas=*(FsGui3DMainCanvas *)appPtr;
	if(NULL!=canvas.slHd)
	{
		auto &shl=*(canvas.slHd);
		YsShellPolygonStore newSelPlg(shl.Conv());

		auto selPlHd=shl.GetSelectedPolygon();
		for(auto plHd : selPlHd)
		{
			if(YSTRUE!=newSelPlg.IsIncluded(plHd))
			{
				auto connPlHd=YsShellExt_TrackingUtil::MakeEdgeConnectedPolygonGroup(shl.Conv(),plHd,NULL);
				newSelPlg.AddPolygon(connPlHd);
			}
		}

		selPlHd=newSelPlg.GetArray();
		shl.SelectPolygon(selPlHd);

		canvas.needRemakeDrawingBuffer|=NEED_REMAKE_DRAWING_SELECTED_POLYGON;
		canvas.SetNeedRedraw(YSTRUE);
	}
}

/*static*/ void FsGui3DMainCanvas::Select_PolygonUsingAtLeastOneOfSelectedVertex(void *appPtr,FsGuiPopUpMenu *,FsGuiPopUpMenuItem *)
{
	auto &canvas=*(FsGui3DMainCanvas *)appPtr;
	if(NULL!=canvas.slHd)
	{
		auto &shl=*(canvas.slHd);
		auto selVtHd=shl.GetSelectedVertex();

		YsShellPolygonStore plg(shl.Conv());
		for(auto vtHd : selVtHd)
		{
			auto vtPlHd=shl.FindPolygonFromVertex(vtHd);
			plg.AddPolygon(vtPlHd);
		}

		shl.SelectPolygon(plg.GetArray());

		canvas.needRemakeDrawingBuffer|=NEED_REMAKE_DRAWING_SELECTED_POLYGON;
		canvas.SetNeedRedraw(YSTRUE);
	}
}
