/* ////////////////////////////////////////////////////////////

  File Name: ysshellext_boundaryutil.h
  Copyright (c) 2014 Soji Yamakawa.  All rights reserved.
  http://www.ysflight.com

//////////////////////////////////////////////////////////// */

#ifndef YSSHELLEXT_BOUNDARYUTIL_IS_INCLUDED
#define YSSHELLEXT_BOUNDARYUTIL_IS_INCLUDED
/* { */

#include <ysshellext.h>

class YsShellExt_BoundaryInfo
{
private:
	class BoundaryContour
	{
	public:
		YsArray <YsShellVertexHandle> vtHdArray;
		void Initialize(void){}
	};

protected:
	YsArray <YsShellVertexHandle> srcEdVtHd;
	YsFixedLengthToMultiHashTable <YsShellPolygonHandle,2,1> edgeToPlgHash;
	YsShellEdgeStore visited;

	YsArray <YsShellVertexHandle> nonDuplicateVtHdArray;
	YsArray <BoundaryContour> contourCache;

public:
	YsShellExt_BoundaryInfo();
	~YsShellExt_BoundaryInfo();
	void CleanUp(void);

	void MakeInfo(
	    const YsShell &shl,
	    YSSIZE_T nPl,const YsShellPolygonHandle plHdArray[]);

	template <const int N,const int M>
	void MakeInfo(
	    const YsShell &shl,
	    const YsArray <YsShellPolygonHandle,N> &plHdArray);

	template <const int N>
	void MakeInfo(
	    const YsShell &shl,
	    const YsArray <YsShellPolygonHandle,N> &plHdArray);

	/*! This function makes a hash-table for vertex connections. */
	void MakeVertexConnectionTable(
		YsFixedLengthToMultiHashTable <YsShellVertexHandle,1,2> &vtKeyToConnVtHd,
		const YsShell &shl) const;

	/*! This function caches the vertex handles that are included in the boundary. */
	void CacheNonDuplicateVtHdArray(void);

	/*! This function must be called after vertices are cached by MakeNonDuplicateVtHdArray. */
	YsVec3 GetCenter(const YsShellExt &shl) const;

	/*! This function must be called after vertices are cached by MakeNonDuplicateVtHdArray. */
	template <const int N>
	void GetVertexAll(YsArray <YsShellVertexHandle,N> &vtHdArray) const;

	/*! This function must be called after vertices are cached by MakeNonDuplicateVtHdArray. */
	const YsArray <YsShellVertexHandle> &GetVertexAll(void) const;

	/*! This function caches countours.  If the contours won't make unique loops, this function will return YSERR. */
	YSRESULT CacheContour(const YsShell &shl);

private:
	YSRESULT TrackAndAddLoop(
	    YsTwoKeyStore &usedEdgePiece,
	    const YsShell &shl,YsShellVertexHandle fromVtHd,YsShellVertexHandle toVtHd,
	    const YsFixedLengthToMultiHashTable <YsShellVertexHandle,1,2> &connection);

public:
	/*! This function returns number of contours.  If contours are not cached, CacheContour function failed, or there is no contour, this function will return zero. */
	YSSIZE_T GetNumContour(void) const;

	/*! This function returns a contour.  idx must be 0<=N<GetNumContour().  Returns YSERR if N is out of bound.  Contours must be cached before this function. */
	YSRESULT GetContour(YSSIZE_T &nVt,const YsShellVertexHandle *&vtHdArray,YSSIZE_T idx) const;

	/*! This function returns a contour.  idx must be 0<=N<GetNumContour().  Returns YSERR and the array will be empty if N is out of bound.  Contours must be cached before this function. 
	    First and last vertex handle in the array will be the same. */
	template <const int N>
	YSRESULT GetContour(YsArray <YsShellVertexHandle,N> &vtHdArray,YSSIZE_T idx) const;

	/*! This function returns arrays of the vertices of the boundary contours. 
	    Must be called after CacheContour.  
	    First and last vertex handles in each array will be the same. */
	YsArray <YsArray <YsShellVertexHandle> > GetContourAll(void) const;

	void GetEdgeUsageMinMax(int &min,int &max,const YsShellExt &shl) const;

	YSSIZE_T GetNumEdgePiece(void) const;
	YSRESULT GetEdge(YsShellVertexHandle edVtHd[2],YSSIZE_T edIdx) const;
};

template <const int N,const int M>
void YsShellExt_BoundaryInfo::MakeInfo(
    const YsShell &shl,
    const YsArray <YsShellPolygonHandle,N> &plHdArray)
{
	MakeInfo(shl,plHdArray.GetN(),plHdArray);
}

template <const int N>
void YsShellExt_BoundaryInfo::MakeInfo(
    const YsShell &shl,
    const YsArray <YsShellPolygonHandle,N> &plHdArray)
{
	MakeInfo <N,N> (shl,plHdArray);
}

template <const int N>
void YsShellExt_BoundaryInfo::GetVertexAll(YsArray <YsShellVertexHandle,N> &vtHdArray) const
{
	vtHdArray=nonDuplicateVtHdArray;
}

template <const int N>
YSRESULT YsShellExt_BoundaryInfo::GetContour(YsArray <YsShellVertexHandle,N> &vtHdArray,YSSIZE_T idx) const
{
	YSSIZE_T n;
	const YsShellVertexHandle *arrayPtr;
	if(YSOK==GetContour(n,arrayPtr,idx))
	{
		vtHdArray.Set(n,arrayPtr);
		return YSOK;
	}
	vtHdArray.CleanUp();
	return YSERR;
}

/* } */
#endif
