/* ////////////////////////////////////////////////////////////

  File Name: ysshellextedit_topologyutil.cpp
  Copyright (c) 2014 Soji Yamakawa.  All rights reserved.
  http://www.ysflight.com

//////////////////////////////////////////////////////////// */

#include "ysshellextedit_topologyutil.h"


void YsShellExtEdit_TopologyUtil::CleanUp(void)
{
	modifiedPlHdArray.CleanUp();
	modifiedCeHdArray.CleanUp();
}


YSRESULT YsShellExtEdit_TopologyUtil::InsertVertexOnEdge(YsShellExtEdit &shl,YsShellVertexHandle fromVtHd,YsShellVertexHandle toVtHd,int nVt,const YsShellVertexHandle vtHdArray[])
{
	CleanUp();

	YsShellExtEdit::StopIncUndo incUndo(&shl);

	YsArray <YsShellPolygonHandle> plHdArray;

	if(YSOK==shl.FindPolygonFromEdgePiece(plHdArray,fromVtHd,toVtHd))
	{
		for(auto plHd : plHdArray)
		{
			YsArray <YsShellVertexHandle,16> plVtHd,newPlVtHd;

			shl.GetPolygon(plVtHd,plHd);
			for(YSSIZE_T vtIdx=0; vtIdx<plVtHd.GetN(); ++vtIdx)
			{
				newPlVtHd.Append(plVtHd[vtIdx]);
				if(fromVtHd==plVtHd[vtIdx] && toVtHd==plVtHd.GetCyclic(vtIdx+1))
				{
					newPlVtHd.Append(nVt,vtHdArray);
				}
				else if(toVtHd==plVtHd[vtIdx] && fromVtHd==plVtHd.GetCyclic(vtIdx+1))
				{
					for(int i=nVt-1; 0<=i; --i)
					{
						newPlVtHd.Append(vtHdArray[i]);
					}
				}
			}

			if(plVtHd.GetN()<newPlVtHd.GetN())
			{
				shl.SetPolygonVertex(plHd,newPlVtHd);
				modifiedPlHdArray.Append(plHd);
			}
		}
	}

	YsArray <YsShellExt::ConstEdgeHandle> ceHdArray;
	if(YSOK==shl.FindConstEdgeFromEdgePiece(ceHdArray,fromVtHd,toVtHd))
	{
		for(auto ceHd : ceHdArray)
		{
			YSBOOL isLoop;
			YsArray <YsShellVertexHandle,16> ceVtHd,newCeVtHd;

			shl.GetConstEdge(ceVtHd,isLoop,ceHd);

			const int nEnd=(YSTRUE==isLoop ? ceVtHd.GetN() : ceVtHd.GetN()-1);

			for(YSSIZE_T vtIdx=0; vtIdx<nEnd; ++vtIdx)
			{
				newCeVtHd.Append(ceVtHd[vtIdx]);
				if(fromVtHd==ceVtHd[vtIdx] && toVtHd==ceVtHd.GetCyclic(vtIdx+1))
				{
					newCeVtHd.Append(nVt,vtHdArray);
				}
				else if(toVtHd==ceVtHd[vtIdx] && fromVtHd==ceVtHd.GetCyclic(vtIdx+1))
				{
					for(int i=nVt-1; 0<=i; --i)
					{
						newCeVtHd.Append(vtHdArray[i]);
					}
				}
			}

			if(YSTRUE!=isLoop)
			{
				newCeVtHd.Append(ceVtHd.Last());
			}

			if(ceVtHd.GetN()<newCeVtHd.GetN())
			{
				shl.ModifyConstEdge(ceHd,newCeVtHd,isLoop);
				modifiedCeHdArray.Append(ceHd);
			}
		}
	}

	return YSOK;
}


YSRESULT YsShellExtEdit_TopologyUtil::SplitPolygon(YsShellExtEdit &shl,YsShellPolygonHandle plHd,YsShellVertexHandle vtHd0,YsShellVertexHandle vtHd1)
{
	CleanUp();

	YsArray <YsShellVertexHandle,16> plVtHd,newPlVtHd;

	shl.GetPolygon(plVtHd,plHd);
	YSSIZE_T vtIdx0=-1,vtIdx1=-1;
	for(YSSIZE_T vtIdx=0; vtIdx<plVtHd.GetN(); ++vtIdx)
	{
		if(plVtHd[vtIdx]==vtHd0)
		{
			vtIdx0=vtIdx;
		}
		else if(plVtHd[vtIdx]==vtHd1)
		{
			vtIdx1=vtIdx;
		}
	}

	if(0<=vtIdx0 && 0<=vtIdx1 && YSOK==YsShellExt_SplitLoop(plVtHd,newPlVtHd,vtIdx0,vtIdx1) && 3<=plVtHd.GetN() && 3<=newPlVtHd.GetN())
	{
		shl.SetPolygonVertex(plHd,plVtHd);

		const YsShellExt::PolygonAttrib *plAtt=shl.GetPolygonAttrib(plHd);
		const YsVec3 nom=shl.GetNormal(plHd);
		const YsColor col=shl.GetColor(plHd);

		YsShellPolygonHandle newPlHd=shl.AddPolygon(newPlVtHd);
		shl.SetPolygonAttrib(newPlHd,*plAtt);
		shl.SetPolygonNormal(newPlHd,nom);
		shl.SetPolygonColor(newPlHd,col);

		modifiedPlHdArray.Append(plHd);
		modifiedPlHdArray.Append(newPlHd);

		return YSOK;
	}
	return YSERR;
}

YSRESULT YsShellExtEdit_TopologyUtil::SplitPolygon(YsShellExtEdit &shl,YsShellPolygonHandle plHd,const YsShellVertexHandle vtHd[2])
{
	return SplitPolygon(shl,plHd,vtHd[0],vtHd[1]);
}
