#ifndef YSGLSL_DDRAWING_IS_INCLUDED
#define YSGLSL_DDRAWING_IS_INCLUDED
/* { */

#include "ysgldef.h"
#include "ysglheader.h"

/* Force Visual C++ to type-mismatching error. */
#pragma warning( error : 4028)
#pragma warning( error : 4047)

#ifdef __cplusplus
extern "C" {
#endif

struct YsGLSL3DRenderer;

/*! This function deletes a 3D rendering object created by YsGLSLCreateXXXX3DRenderer. 
*/
void YsGLSLDelete3DRenderer(struct YsGLSL3DRenderer *renderer);

/*! This function sets the projection matrix to the 3D renderer program. 
    The 3D rendere program must be current.  Use YsGLSLUse3DRenderer to make it current.  
*/
void YsGLSLSet3DRendererProjectionfv(struct YsGLSL3DRenderer *renderer,const GLfloat mat[16]);

/*! This function sets the projection matrix to the 3D renderer program. 
    The 3D rendere program must be current.  Use YsGLSLUse3DRenderer to make it current.  
*/
void YsGLSLSet3DRendererProjectiondv(struct YsGLSL3DRenderer *renderer,const double mat[16]);

/*! This function gets the projection matrix from the 3D renderer program. 
    Since the matrix is cached, the 3D rendere program does not have to be current.
*/
void YsGLSLGet3DRendererProjectionfv(GLfloat mat[16],const struct YsGLSL3DRenderer *renderer);

/*! This function gets the projection matrix from the 3D renderer program. 
    Since the matrix is cached, the 3D rendere program does not have to be current.
*/
void YsGLSLGet3DRendererProjectiondv(double mat[16],const struct YsGLSL3DRenderer *renderer);

/*! This function sets the model view matrix to the 3D renderer program. 
    The 3D rendere program must be current.  Use YsGLSLUse3DRenderer to make it current.  
*/
void YsGLSLSet3DRendererModelViewfv(struct YsGLSL3DRenderer *renderer,const GLfloat mat[16]);

/*! This function sets the model view matrix to the 3D renderer program. 
    The 3D rendere program must be current.  Use YsGLSLUse3DRenderer to make it current.  
*/
void YsGLSLSet3DRendererModelViewdv(struct YsGLSL3DRenderer *renderer,const double mat[16]);

/*! This function gets the model view matrix from the 3D renderer program. 
    Since the matrix is cached, the 3D rendere program does not have to be current.
*/
void YsGLSLGet3DRendererModelViewfv(GLfloat mat[16],const struct YsGLSL3DRenderer *renderer);

/*! This function gets the model view matrix from the 3D renderer program. 
    Since the matrix is cached, the 3D rendere program does not have to be current.
*/
void YsGLSLGet3DRendererModelViewdv(double mat[16],const struct YsGLSL3DRenderer *renderer);

/*! This function sets texture type.  Texture type must be one of the following.
    YSGLSL_TEX_TYPE_NONE
    YSGLSL_TEX_TYPE_TILING
    YSGLSL_TEX_TYPE_BILLBOARD
*/
YSRESULT YsGLSLSet3DRendererTextureType(const struct YsGLSL3DRenderer *renderer,int textureType);

/*! This function sets the texture identifier for tiling and billboarding. 
    The 3D rendere program must be current.  Use YsGLSLUse3DRenderer to make it current.  

    Surprisingly, GL_TEXTURE0 is not equal to int(0).  So, do not give GL_TEXTURE0, but give 0 as textureIdent.
*/
YSRESULT YsGLSLSet3DRendererTextureIdentifier(const struct YsGLSL3DRenderer *renderer,GLuint textureIdent);

/*! This function sets the texture-tiling transformation.  Incoming coordinate (x,y,z) will be transformed by 
    the matrix to calculate (u,v).  (u=x', v=y')
    The 3D rendere program must be current.  Use YsGLSLUse3DRenderer to make it current.  
    */
YSRESULT YsGLSLSet3DRendererUniformTextureTilingMatrixfv(const struct YsGLSL3DRenderer *renderer,const GLfloat texTfm[16]);

/*! This function sets Bill board clipping type.  Currently virtual lighting is only available in BillBoardRenderer. 
    Virtual-lighting type can be one of:
      YSGLSL_BILLBOARD_CLIP_NONE
      YSGLSL_BILLBOARD_CLIP_CIRCLE
    */
YSRESULT YsGLSLSet3DRendererBillBoardClippingType(const struct YsGLSL3DRenderer *renderer,int billBoardClippingType);

/*! This function sets Bill board transformation type.  Bill board transformation type must be one of the following..
    YSGLSL_TEX_BILLBOARD_PERS
    YSGLSL_TEX_BILLBOARD_ORTHO
 */
YSRESULT YsGLSLSet3DRendererBillBoardTransformType(const struct YsGLSL3DRenderer *renderer,int billBoardTfmType);

/*! This function sets the center and the dimension of billboard. */
YSRESULT YsGLSLSet3DRendererBillBoardfv(const struct YsGLSL3DRenderer *renderer,const GLfloat center[3],const GLfloat dimension[2]);

/*! This function sets the uniform color to the 3D renderer program.
    This function has effect on:
        Flat3DRenderer
    If the given renderer is not one of these, it returns YSERR.
*/
YSRESULT YsGLSLSet3DRendererUniformColorfv(struct YsGLSL3DRenderer *renderer,const GLfloat color[4]);

/*! This function sets viewport dimension to the 3D rendere program.
    This function has effect on:
        MonoColorMarker3DRenderer
        VariColorMarker3DRenderer
    If the given renderer is not one of these, it returns YSERR.
*/
YSRESULT YsGLSLSet3DRendererViewportDimensionf(const struct YsGLSL3DRenderer *renderer,const GLfloat width,const GLfloat height);

/*! This function sets alpha-cutoff to the 3D renderer program. */
YSRESULT YsGLSLSet3DRendererAlphaCutOff(const struct YsGLSL3DRenderer *renderer,GLfloat alphaCutOff);

/*! This function sets light direction.  
    The 3D rendere program must be current.  Use YsGLSLUse3DRenderer to make it current.

    Lighting is calculated in the camera coordinate, therefore the light is transformed by the current model view matrix 
    before sent to the shader program.
    */
YSRESULT YsGLSLSet3DRendererUniformLightDirectionfv(struct YsGLSL3DRenderer *renderer,int idx,const GLfloat dir[3]);

/*! This function sets light color. */
YSRESULT YsGLSLSet3DRendererLightColor(struct YsGLSL3DRenderer *renderer,int idx,const GLfloat color[3]);

/*! This function sets ambient-light color. */
YSRESULT YsGLSLSet3DRendererAmbientColor(struct YsGLSL3DRenderer *renderer,const GLfloat color[3]);

/*! This function sets specular-reflection color. */
YSRESULT YsGLSLSet3DRendererSpecularColor(struct YsGLSL3DRenderer *renderer,const GLfloat color[3]);

/*! This function sets specular-reflection exponent. */
YSRESULT YsGLSLSet3DRendererSpecularExponent(struct YsGLSL3DRenderer *renderer,const GLfloat exponent);

/*! This function sets the size of the flash drawn.  Equivalent to YsGLSLSet3DRendererUniformMarkerDimension. */
YSRESULT YsGLSLSet3DRendererUniformFlashSize(struct YsGLSL3DRenderer *renderer,GLfloat flashSize);

/*! This function sets the relative radii of flash. */
YSRESULT YsGLSLSet3DRendererFlashRadius(struct YsGLSL3DRenderer *renderer,GLfloat r1,GLfloat r2);

/*! This function turn on/off the fog. */
YSRESULT YsGLSLSet3DRendererUniformFogEnabled(struct YsGLSL3DRenderer *renderer,int enableFog);

/*! This function sets fog density. */
YSRESULT YsGLSLSet3DRendererUniformFogDensity(struct YsGLSL3DRenderer *renderer,const GLfloat density);

/*! This function sets fog color. */
YSRESULT YsGLSLSet3DRendererUniformFogColor(struct YsGLSL3DRenderer *renderer,const GLfloat fogColor[4]);

/*! This function returns fog density set in the renderer. */
GLfloat YsGLSLGet3DRendererFogDensity(struct YsGLSL3DRenderer *renderer);

/*! This function turns on/off the z offset. */
YSRESULT YsGLSLSet3DRendererZOffsetEnabled(struct YsGLSL3DRenderer *renderer,int enableZOffset);

/*! This function sets z offset. */
YSRESULT YsGLSLSet3DRendererZOffset(struct YsGLSL3DRenderer *renderer,GLfloat zOffset);

/*! This function makes the shader program of the renderer current, and returns the previous
    current program ID. Once a 3DRenderer is activated by this function, the program must end using
    the renderer by using YsGLSLEndUse3DRenderer function.
    
    In some implementation, attribute array state (enabled/disabled) is not per-program, and leaving an
    attribute array enabled and switching to a different program may cause a segmentation fault.
*/
GLuint YsGLSLUse3DRenderer(struct YsGLSL3DRenderer *renderer);

/*! This function ends using 3D renderer activated by YsGLSLUse3DRenderer function.  A renderer must 
    always be de-activated by this function once it is activated.
    This function does nothing if renderer is NULL.
*/
void YsGLSLEndUse3DRenderer(struct YsGLSL3DRenderer *renderer);

/*! This function returns non-zero if the renderer is in-use by YsGLSLUse3DRenderer. */
int YsGLSLCheck3DRendererIsInUse(struct YsGLSL3DRenderer *renderer);


/*! This function applies sphere map to the bitmap with the lighting stored in the renderer. 
    If useSpecular is non zero, specular reflection will be calculated assuming that the sphere is infinitely far in the depth direction. */
int YsGLSL3DRendererApplySphereMap(int w,int h,GLfloat *rgba,struct YsGLSL3DRenderer *renderer,int useSpecular);



/*! This function creates a 3D flat-renderer. 
*/
struct YsGLSL3DRenderer *YsGLSLCreateFlat3DRenderer(void);

/*! This function draws primitives.  The current renderer must be Flat3DRenderer.
*/
YSRESULT YsGLSLDrawPrimitiveVtxfv(const struct YsGLSL3DRenderer *renderer,GLenum mode,GLsizei nVtx,const GLfloat vtx[]);

/*! This function creates a mono-color bill-board renderer.  */
struct YsGLSL3DRenderer *YsGLSLCreateMonoColorBillBoard3DRenderer(void);

/*! This function creates a vari-color bill board renderer. */
struct YsGLSL3DRenderer *YsGLSLCreateVariColorBillBoard3DRenderer(void);

/*! This function draws a primitive with vertex, color, view-offset, texture-coordinate, as attributes.
    vOffset is an array of vec3s defining offset vectors in the view coordinate.
    texCoord is an array of vec2s defining texture coordinates for the vertex. */
YSRESULT YsGLSLDrawPrimitiveVtxColVoffsetTexcoordfv(
    const struct YsGLSL3DRenderer *renderer,GLenum mode,
    GLsizei nVtx,
    const GLfloat vtx[],     // 3*nVtx
    const GLfloat col[],     // 4*nVtx
    const GLfloat vOffset[], // 3*nVtx   Let it modify Z as well
    const GLfloat texCoord[] // 2*nVtx
    );


/*! This function draws a primitive with vertex, view-offset, texture-coordinate, as attributes.
    vOffset is an array of vec3s defining offset vectors in the view coordinate.
    texCoord is an array of vec2s defining texture coordinates for the vertex. */
YSRESULT YsGLSLDrawPrimitiveVtxVoffsetTexcoordfv(
    const struct YsGLSL3DRenderer *renderer,GLenum mode,
    GLsizei nVtx,
    const GLfloat vtx[],     // 3*nVtx
    const GLfloat vOffset[], // 3*nVtx   Let it modify Z as well
    const GLfloat texCoord[] // 2*nVtx
    );


/*! This function creates a 3D flash-renderer.
 */
struct YsGLSL3DRenderer *YsGLSLCreateFlash3DRenderer(void);

/*! This function draws primitives.  The current renderer must be Flash3DRenderer.
    The vertex and viewOffset must draw a set of GL_TRIANGLES that covers the area of a flash.
 */
YSRESULT YsGLSLDrawPrimitiveVtxColVoffsetfv(const struct YsGLSL3DRenderer *renderer,GLsizei nVtx,const GLfloat vtx[],const GLfloat col[],const GLfloat viewOffset[]);

/*! This function creates a mono-color marker renderer. */
struct YsGLSL3DRenderer *YsGLSLCreateMonoColorMarker3DRenderer(void);

/*! This function sets the size of the flash drawn.  Equivalent to YsGLSLSet3DRendererUniformFlashSize. 
    The renderer needs to be MonoColorMarker3DRenderer or VariColorMarker3DRenderer. */
YSRESULT YsGLSLSet3DRendererUniformMarkerDimension(struct YsGLSL3DRenderer *renderer,GLfloat dimension);

/*! This function sets the marker type.  The renderer must be current and of one of:
      MonoColorMarker3DRenderer or VariColorMarker3DRenderer. */
YSRESULT YsGLSLSet3DRendererUniformMarkerType(struct YsGLSL3DRenderer *renderer,int markerType);

/*! This function draws primitives.  The current renderer must be MonoColorMarker3DRenderer.
 */
YSRESULT YsGLSLDrawPrimitiveVtxPixelOffsetfv(const struct YsGLSL3DRenderer *renderer,GLenum mode,GLsizei nVtx,const GLfloat vtx[],const GLfloat pixelOffset[]);

/*! This function creates a vari-color marker renderer. */
struct YsGLSL3DRenderer *YsGLSLCreateVariColorMarker3DRenderer(void);

/*! This function draws primitives.  The current renderer must be VariColorMarker3DRenderer. */
YSRESULT YsGLSLDrawPrimitiveVtxColPixelOffsetfv(const struct YsGLSL3DRenderer *renderer,GLenum mode,GLsizei nVtx,const GLfloat vtx[],const GLfloat col[],const GLfloat pixelOffset[]);

/*! This function creates a 3D variable-color no-lighting renderer.
*/
struct YsGLSL3DRenderer *YsGLSLCreateVariColor3DRenderer(void);

/*! This function draws primitivs with color on each vertex.  Each color consists of four components, R,G,B, and A.  
    The length of array col thus must be nVtx*4.
    The current renderer must be VariColor3DRenderer.
*/
YSRESULT YsGLSLDrawPrimitiveVtxColfv(const struct YsGLSL3DRenderer *renderer,GLenum mode,GLsizei nVtx,const GLfloat vtx[],const GLfloat col[]);

/*! This function creates a 3D monotonic-color per-vertex shading renderer. */
struct YsGLSL3DRenderer *YsGLSLCreateMonoColorPerVtxShading3DRenderer(void);

/*! This function creates a 3D monotonic-color per-pixel shading renderer with tex coord as attribute. 
    Note.  The population explosion of the shaders could be stopped if I could use generic attributes.
    But, OpenGL on Mac OSX does not support generic attributes (it is broken.)  See comments in ysglslutil.c
*/
struct YsGLSL3DRenderer *YsGLSLCreateMonoColorPerVtxShadingWithTexCoord3DRenderer(void);

/*! This function draws primitives.  The current renderer must be one of:
    MonoColorPerVtxShading3DRenderer
    MonoColorPerPixShading3DRenderer
    VariColorPerVtxShading3DRenderer
    VariColorPerPixShading3DRenderer

    Color value from the most recent call of YsGLSLSet3DRendererUniformColorfv will be used.
*/
YSRESULT YsGLSLDrawPrimitiveVtxNomfv(const struct YsGLSL3DRenderer *renderer,GLenum mode,GLsizei nVtx,const GLfloat vtx[],const GLfloat nom[]);

/*! This function draws primitives.  The current renderer must be one of:
    MonoColorPerVtxShading3DRenderer
    MonoColorPerPixShading3DRenderer
    VariColorPerVtxShading3DRenderer
    VariColorPerPixShading3DRenderer

    Color value from the most recent call of YsGLSLSet3DRendererUniformColorfv will be used.

	Texture type must be set to YSGLSL_TEX_TYPE_ATTRIBUTE, or the texCoord will be ignored.
*/
YSRESULT YsGLSLDrawPrimitiveVtxTexCoordNomfv(const struct YsGLSL3DRenderer *renderer,GLenum mode,GLsizei nVtx,const GLfloat vtx[],const GLfloat texCoord[],const GLfloat nom[]);

/*! This function creates a 3D monotonic-color per-pixel shading renderer.
*/
struct YsGLSL3DRenderer *YsGLSLCreateMonoColorPerPixShading3DRenderer(void);

/*! This function creates a 3D monotonic-color per-pixel shading renderer with tex coord as attribute. 
    Note.  The population explosion of the shaders could be stopped if I could use generic attributes.
    But, OpenGL on Mac OSX does not support generic attributes (it is broken.)  See comments in ysglslutil.c
*/
struct YsGLSL3DRenderer *YsGLSLCreateMonoColorPerPixShadingWithTexCoord3DRenderer(void);

/*! This function creates a 3D varying-color per-vertex shading renderer. 
*/
struct YsGLSL3DRenderer *YsGLSLCreateVariColorPerVtxShading3DRenderer(void);

/*! This function draws primitives with normal and color per vertex.
    The current renderer must be one of:
      VariColorPerVtxShading3DRenderer
      VariColorPerPixShading3DRenderer
      MonoColorPerVtxShading3DRenderer
      MonoColorPerPixShading3DRenderer
    If the renderer is a monotonic-color renderer, the first color is used for drawing all the primitives. 
    The length of vtx must be 3*nVtx.
    The length of nom must be 3*nVtx.
    The length of col must be 4*nVtx (including alpha).
*/
YSRESULT YsGLSLDrawPrimitiveVtxNomColfv(const struct YsGLSL3DRenderer *renderer, GLenum mode,GLsizei nVtx,const GLfloat vtx[],const GLfloat nom[],const GLfloat col[]);

/*! This function draws primitives with normal and color per vertex.
    The current renderer must be one of:
      VariColorPerVtxShading3DRenderer
      VariColorPerPixShading3DRenderer
      MonoColorPerVtxShading3DRenderer
      MonoColorPerPixShading3DRenderer
    If the renderer is a monotonic-color renderer, the first color is used for drawing all the primitives. 
    The length of vtx must be 3*nVtx.
    The length of texCoord must be 2*nVtx
    The length of nom must be 3*nVtx.
    The length of col must be 4*nVtx (including alpha).
*/
YSRESULT YsGLSLDrawPrimitiveVtxTexCoordNomColfv(const struct YsGLSL3DRenderer *renderer, GLenum mode,GLsizei nVtx,const GLfloat vtx[],const GLfloat texCoord[],const GLfloat nom[],const GLfloat col[]);

/*! This function creates a 3D varying-color per-pixel shading renderer.
*/
struct YsGLSL3DRenderer *YsGLSLCreateVariColorPerPixShading3DRenderer(void);


/*! This function creates a 3D varying-color per-pixel shading renderer with tex coord as attribute. 
    Note.  The population explosion of the shaders could be stopped if I could use generic attributes.
    But, OpenGL on Mac OSX does not support generic attributes (it is broken.)  See comments in ysglslutil.c
*/
struct YsGLSL3DRenderer *YsGLSLCreateVariColorPerPixShadingWithTexCoord3DRenderer(void);


/*! This function creates a 3D varying-color per-vertex shading renderer with tex coord as attribute. 
    Note.  The population explosion of the shaders could be stopped if I could use generic attributes.
    But, OpenGL on Mac OSX does not support generic attributes (it is broken.)  See comments in ysglslutil.c
*/
struct YsGLSL3DRenderer *YsGLSLCreateVariColorPerVtxShadingWithTexCoord3DRenderer(void);


/* Will need

New Uniforms:
  Texture Identifier
  Repeat switch

struct YsGLSL3DRenderer *YsGLSLCreateVariColorWithTex3DRenderer(void);
YSRESULT YsGLSLDrawPrimitiveVtxColTexfv
YSRESULT YsGLSLDrawPrimitiveVtxNomColTexfv
*/

#ifdef __cplusplus
}
#endif

/* } */
#endif
