#ifndef YSGLSLSHAREDRENDERER_IS_INCLUDED
#define YSGLSLSHAREDRENDERER_IS_INCLUDED
/* { */

/* Force Visual C++ to type-mismatching error. */
#pragma warning( error : 4028)
#pragma warning( error : 4047)

#ifdef __cplusplus
extern "C" {
#endif

#include "ysgldef.h"
#include "ysglsl3ddrawing.h"
#include "ysglslplain2ddrawing.h"
#include "ysglslbitmaprenderer.h"

/*! This function allocates shared renderers.
*/
YSRESULT YsGLSLCreateSharedRenderer(void);

/*! This function de-allocates shared renderes.
*/
YSRESULT YsGLSLDeleteSharedRenderer(void);

/*! This function returns a pointer to the shared Flat3DRenderer. */
struct YsGLSL3DRenderer *YsGLSLSharedFlat3DRenderer(void);

/*! This function returns a pointer to the shared vari-color bill-board 3D renderer. */
struct YsGLSL3DRenderer *YsGLSLSharedVariColorBillBoard3DRenderer(void);

/*! This function returns a pointer to the shared mono-color bill-board 3D renderer. */
struct YsGLSL3DRenderer *YsGLSLSharedMonoColorBillBoard3DRenderer(void);

/*! This function returns a pointer to the shared FlatshDRenderer. */
struct YsGLSL3DRenderer *YsGLSLSharedFlash3DRenderer(void);

/*! This function returns a pointer to the shared VariColor3DRenderer */
struct YsGLSL3DRenderer *YsGLSLSharedVariColor3DRenderer(void);

/*! This function returns a pointer to the shared MonoColorPerVtxShading3DRenderer */
struct YsGLSL3DRenderer *YsGLSLSharedMonoColorPerVtxShading3DRenderer(void);

/*! This function returns a pointer to the shared MonoColorPerPixShading3DRenderer */
struct YsGLSL3DRenderer *YsGLSLSharedMonoColorPerPixShading3DRenderer(void);

/*! This function returns a pointer to the shared VariColorPerVtxShading3DRenderer */
struct YsGLSL3DRenderer *YsGLSLSharedVariColorPerVtxShading3DRenderer(void);

/*! This function returns a pointer to the shared VariColorPerPixShading3DRenderer */
struct YsGLSL3DRenderer *YsGLSLSharedVariColorPerPixShading3DRenderer(void);

/*! This function returns a pointer to the shared MonoColorPerVtxShading3DRenderer */
struct YsGLSL3DRenderer *YsGLSLSharedMonoColorPerVtxShadingWithTexCoord3DRenderer(void);

/*! This function returns a pointer to the shared VariColorPerVtxShading3DRenderer */
struct YsGLSL3DRenderer *YsGLSLSharedVariColorPerVtxShadingWithTexCoord3DRenderer(void);

/*! This function returns a pointer to the shared MonoColorPerPixShading3DRenderer */
struct YsGLSL3DRenderer *YsGLSLSharedMonoColorPerPixShadingWithTexCoord3DRenderer(void);

/*! This function returns a pointer to the shared VariColorPerPixShading3DRenderer */
struct YsGLSL3DRenderer *YsGLSLSharedVariColorPerPixShadingWithTexCoord3DRenderer(void);

/*! This runction returns a pointer to the shared MonoColorMarker3DRendere */
struct YsGLSL3DRenderer *YsGLSLSharedMonoColorMarker3DRenderer(void);

/*! This runction returns a pointer to the shared MonoColorMarker3DRendere */
struct YsGLSL3DRenderer *YsGLSLSharedVariColorMarker3DRenderer(void);

/*! This function sets alpha-cutoff value to the shared 3D renderers. */
YSRESULT YsGLSLSetShared3DRendererAlphaCutOff(GLfloat alphaCutOff);

/*! This function sets fog to all applicable shared 3D renderers. */
YSRESULT YsGLSLSetShared3DRendererFog(int enableFog,const GLfloat density,const GLfloat fogColor[4]);

/*! This function sets projection matrix to all applicable shared 3D renderers. */
YSRESULT YsGLSLSetShared3DRendererProjection(const GLfloat proj[16]);

/*! This function sets modelview matrix to all applicable shared 3D renderers. */
YSRESULT YsGLSLSetShared3DRendererModelView(const GLfloat modelview[16]);

/*! This function sets directional light to all applicable shared 3D renderers. */
YSRESULT YsGLSLSetShared3DRendererDirectionalLightfv(int i,const GLfloat dir[3]);

/*! This function sets light color to all applicable shared 3D renderers. */
YSRESULT YsGLSLSetShared3DRendererLightColor(int idx,const GLfloat color[3]);

/*! This function sets ambient-light color to all applicable shared 3D renderers. */
YSRESULT YsGLSLSetShared3DRendererAmbientColor(const GLfloat color[3]);

/*! This function sets specular reflection color to all applicable shared 3D renderers. */
YSRESULT YsGLSLSetShared3DRendererSpecularColor(const GLfloat color[3]);

/*! This function sets specular exponent to all applicable shared 3D renderers. */
YSRESULT YsGLSLSetShared3DRendererSpecularExponent(const GLfloat exponent);

/*! This function sets the view port dimension to all applicable shared 3D renderers. */
YSRESULT YsGLSLSetShared3DRendererViewPortDimension(const GLfloat wid,const GLfloat hei);


/*! This function returns a pointer to the shared Plain2DRenderer */
struct YsGLSLPlain2DRenderer *YsGLSLSharedPlain2DRenderer(void);

/*! This function returns a pointer to the shared BitmapRenderer */
struct YsGLSLBitmapRenderer *YsGLSLSharedBitmapRenderer(void);


/*! This function controls which renderer is returned by the following three functions:
      YsGLSLSharedMonoColorShaded3DRenderer
      YsGLSLSharedVariColorShaded3DRenderer
    If this flag is set to YSTRUE, the renderer returned by the two functions are:
      YsGLSLSharedMonoColorShaded3DRenderer => YsGLSLSharedMonoColorPerPixShading3DRenderer()
      YsGLSLSharedVariColorShaded3DRenderer => YsGLSLSharedVariColorPerPixShading3DRenderer()
    otherwise,
      YsGLSLSharedMonoColorShaded3DRenderer => YsGLSLSharedMonoColorPerVtxShading3DRenderer()
      YsGLSLSharedVariColorShaded3DRenderer => YsGLSLSharedVariColorPerVtxShading3DRenderer()

    Default value is YSFALSE.
*/
void YsGLSLSetPerPixRendering(YSBOOL perPix);

/*! This function returns YsGLSLSharedMonoColorPerPixShading3DRenderer() or
    YsGLSLSharedMonoColorPerVtxShading3DRenderer() depending on the switch set 
    by YsGLSLSetPerPixRendering. */
struct YsGLSL3DRenderer *YsGLSLSharedMonoColorShaded3DRenderer(void);

/*! This function returns YsGLSLSharedVariColorPerPixShading3DRenderer() or
    YsGLSLSharedVariColorPerVtxShading3DRenderer() depending on the switch set 
    by YsGLSLSetPerPixRendering. */
struct YsGLSL3DRenderer *YsGLSLSharedVariColorShaded3DRenderer(void);


/*! This function returns YsGLSLSharedMonoColorPerPixShadingWithTexCoord3DRenderer() or
    YsGLSLSharedMonoColorPerVtxShadingWithTexCoord3DRenderer() depending on the switch set 
    by YsGLSLSetPerPixRendering. */
struct YsGLSL3DRenderer *YsGLSLSharedMonoColorShadedWithTexCoord3DRenderer(void);

/*! This function returns YsGLSLSharedVariColorPerPixShadingWithTexCoord3DRenderer() or
    YsGLSLSharedVariColorPerVtxShadingWithTexCoord3DRenderer() depending on the switch set 
    by YsGLSLSetPerPixRendering. */
struct YsGLSL3DRenderer *YsGLSLSharedVariColorShadedWithTexCoord3DRenderer(void);


#ifdef __cplusplus
} // extern "C"
#endif

/* } */
#endif
