#include <stdio.h>
#include <yssystemfont.h>

extern "C" unsigned char *YsMacOSXSystemFont_AllocBuffer(size_t bufSize)
{
	unsigned char *buf=new unsigned char [bufSize];
	return buf;
}

extern "C" void *YsMacOSXSystemFont_CreateContext(void);
extern "C" void YsMacOSXSystemFont_FreeContext(void *ctxtPtr);
extern "C" int YsMacOSXSystemFont_SelectSystemFontWithHeight(void *ctxtPtr,unsigned int height);
extern "C" int YsMacOSXSystemFont_SelectSystemFont(void *ctxtPtr);
extern "C" unsigned char *YsMacOSXSystemFont_RGBABitmap(
	void *ctxtPtr,
	unsigned int *wid,unsigned int *hei,unsigned int *dstBytePerLine,
	const wchar_t str[],unsigned int dstBitPerPixel,
	const unsigned char fgCol[3],const unsigned char bgCol[3],
	int reverse);
extern "C" void YsMacOSXSystemFont_GetTightBitmapSize(void *ctxPtr,int *wid,int *hei,const wchar_t str[]);

YsSystemFontCache::YsSystemFontCache()
{
	internal=(YsSystemFontInternal *)YsMacOSXSystemFont_CreateContext();
}

YsSystemFontCache::~YsSystemFontCache()
{
	YsMacOSXSystemFont_FreeContext(internal);
}

YSRESULT YsSystemFontCache::RequestDefaultFont(void)
{
	YsMacOSXSystemFont_SelectSystemFont(internal);
}

YSRESULT YsSystemFontCache::RequestDefaultFontWithHeight(int height)
{
	YsMacOSXSystemFont_SelectSystemFontWithHeight(internal,height);
}

YSRESULT YsSystemFontCache::MakeSingleBitBitmap(YsSystemFontTextBitmap &bmp,const wchar_t wStr[],YSBOOL reverse) const
{
	unsigned int wid,hei,bytePerLine;
	const unsigned int bitPerPixel=1;
	const unsigned char fgCol[3]={255,255,255},bgCol[3]={0,0,0};

	unsigned char *dat=
		YsMacOSXSystemFont_RGBABitmap(internal,&wid,&hei,&bytePerLine,wStr,bitPerPixel,fgCol,bgCol,reverse);
	if(NULL!=dat)
	{
		bmp.SetBitmap(wid,hei,bytePerLine,bitPerPixel,dat);
		return YSOK;
	}
	return YSERR;
}

YSRESULT YsSystemFontCache::MakeRGBABitmap(YsSystemFontTextBitmap &bmp,const wchar_t wStr[],const unsigned char fgCol[3],const unsigned char bgCol[3],YSBOOL reverse) const
{
	unsigned int wid,hei,bytePerLine;
	const unsigned int bitPerPixel=32;

	unsigned char *dat=
		YsMacOSXSystemFont_RGBABitmap(internal,&wid,&hei,&bytePerLine,wStr,bitPerPixel,fgCol,bgCol,reverse);
	if(NULL!=dat)
	{
		bmp.SetBitmap(wid,hei,bytePerLine,bitPerPixel,dat);
		return YSOK;
	}
	return YSERR;
}

YSRESULT YsSystemFontCache::MakeGrayScaleAndAlphaBitmap(YsSystemFontTextBitmap &bmp,const wchar_t wStr[],unsigned char fgColS,unsigned char bgColS,YSBOOL reverse) const
{
	const unsigned char fgCol[3]={fgColS,fgColS,fgColS},bgCol[3]={bgColS,bgColS,bgColS};

	unsigned int wid,hei,bytePerLine;
	const unsigned int bitPerPixel=16;

	unsigned char *dat=
		YsMacOSXSystemFont_RGBABitmap(internal,&wid,&hei,&bytePerLine,wStr,bitPerPixel,fgCol,bgCol,reverse);
	if(NULL!=dat)
	{
		bmp.SetBitmap(wid,hei,bytePerLine,bitPerPixel,dat);
		return YSOK;
	}
	return YSERR;
}

YSRESULT YsSystemFontCache::GetTightBitmapSize(int &wid,int &hei,const wchar_t wStr[]) const
{
	YsMacOSXSystemFont_GetTightBitmapSize(internal,&wid,&hei,wStr);
	return YSOK;
}

